<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00001000ItemLancamentoPropostaService;
use Teknisa\SM\Service\Sm00001000ItemFormPrecoPropostaService;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;

class Sm00001000ItemLancamentoPropostaController extends SMCrud {
    
    protected $dataSourceName       = 'fct_itemorcamento_widget';
    protected $dataSourceNameToSave = 'fct_itemorcamento';
    protected $itensService;
    protected $propostaService;
    protected $itFormacaoService;

    public function __construct ($crudService, Environment $environment, Sm00001000ItemLancamentoPropostaService $itensService, Sm00001000LancamentoPropostaService $propostaService, Sm00001000ItemFormPrecoPropostaService $itFormacaoService){
        parent::__construct($crudService, $environment);
        $this->itensService      = $itensService;
        $this->propostaService   = $propostaService;
        $this->itFormacaoService = $itFormacaoService;
    }
    
    public function find(Request\Filter $request, Response $response){
        try {
            $request->setParameter("dbname", "fct_itemorcamentoarvore_widget");
            parent::find($request, $response);
            
            $rows = $response->getDataSets()[0]->getRows();
            $rows = $this->formatarValoresGrid($rows);
            $response->setDataSets(array(new DataSet("fct_itemorcamento_arvore_medidas_variaveis", $rows)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findItem(Request\Filter $request, Response $response) {
        parent::find($request, $response);
    }
    
    /**
     * @param Request\DataSet $request
     */
    public function saveItem(Request $request, Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        try {
            parent::save($request, $response);
            $erro = $response->getError();
            if ($erro) {
                throw new \Exception($erro->getMessage(), $erro->getErrorCode());
            }
            
            $columnsToReturn = $response->getDataSets()[0]->getRows()[0];
            $nritemorcamento = $columnsToReturn["NRITEMORCAMENTO"];

            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $nrseqorcamento = $row["NRSEQORCAMENTO"];
        
            if ($row["__is_new"]) {
                $row["NRITEMORCAMENTO"] = $nritemorcamento;
                $valorProposta = $this->itensService->inserirItemNegociacaoValoresProposta($row, $nrorg, $cdoperador);
                $response->addDataSet(new DataSet("retorno", array("valorProposta" => $valorProposta)));
                
                $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
                $idGeraRateio    = $this->itFormacaoService->getIdGeraRateioParametros($nrtipoorcamento, $nrorg);
                if(count($idGeraRateio) > 0 && $idGeraRateio[0]["IDGERARATEIO"] == 'S')
                    $this->itFormacaoService->geraRateios(array(), $nrseqorcamento, false, null, $nrtipoorcamento);
            }

            if (isset($row["INSERIR_MEDIDA"]) && $row["INSERIR_MEDIDA"] == true) {
                //$this->itensService->inserirOuAtualizarMedidaVariavelServico($row, $nrorg, $cdoperador);
                $this->propostaService->inserirOuAtualizarMedidaVariavelServico($row, $nrorg, $cdoperador);
            }
            $this->itensService->excluirMedidasVariaveisPropostaSemItemRelacionado($nrseqorcamento, $nrorg);
        }
        catch (\Exception $e) {
            if (isset($nritemorcamento) && $row["__is_new"]) {
                $this->itensService->excluiItemAgendPendente($nrseqorcamento, $nritemorcamento, $nrorg);
                $this->itensService->excluirItemProposta($nritemorcamento, $nrorg);
            }
            
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
    * efetua os calculos dos valores ligados ao item 
    **/
    public function formatarValoresGrid($rows) {
        foreach($rows as &$row){
            if($row["IDTIPO"] == "M"){
                $row["VRUNITITEMORCA_GRID"] = number_format((double)$row["VRUNITITEMORCA_GRID"], 2, ",", ".");
                $row["VRDESCORCA_GRID"]     = number_format((double)$row["VRDESCORCA_GRID"], 2, ",", ".");
                $row["QTMEDIDAORCA_GRID"]   = number_format((double)$row["QTMEDIDAORCA_GRID"], 2, ",", ".");
            }
        }
        return $rows;
    }
     /**
     *Valida se o percentul dado pelo o vendedor esta conforme com seu perfil de acesso ou operador
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function validaPorcentualDescontoAcrescimoSave(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $cdservorca = $row["CDSERVORCA"];
            $idtipo = $row["IDTIPO"];

            $nrorg = $this->environment->getNrOrg();
            
            $cdoperador =$this->environment->getCdOperador();
            $desconto = $this->itensService->validaPorcentualDescontoAcrescimoSave($cdservorca, $cdoperador, $idtipo, $nrorg);
            $response->addDataSet(new DataSet("RESULTADO", array("VRPERCDESC" => $desconto)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Exclui os itens do Agendamento Pendente caso eles sejam excluídos na proposta
    **/
    public function DeletaItemOrcamentoConcluido(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nritemorcamento = $row["NRITEMORCAMENTO"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $cdservorca      = $row["CDSERVORCA"];
            
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->itensService->DeletaItemOrcamentoConcluido($nrseqorcamento, $nritemorcamento, $nrtipoorcamento, $cdservorca, $nrorg, $cdoperador);
            
            $request->setParameter("dbname", $this->dataSourceName);
            parent::delete($request, $response);

            // Atualiza o valor das negociações de valores da proposta com a soma dos itens
            $this->propostaService->atualizarValorTodasNegociacoesComSomaDosItens($nrseqorcamento, $nrorg, $cdoperador);
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function atualizaQuantidadeItensAgendPendente(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nritemorcamento = $row["NRITEMORCAMENTO"];
            $quantidade     = $row["QTPENDENCIA"];
            $nrorg = $this->environment->getNrOrg();
            $this->itensService->atualizaQuantidadeItensAgendPendente($nrseqorcamento, $nritemorcamento, $nrorg, $quantidade);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Replica as alterações de medida ou ajuste para o item pai
    **/
    public function replicaAteracaoParaItemPai(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $cdservorca         = $row["CDSERVORCA"];
            $nrseqorcamento     = $row["NRSEQORCAMENTO"];
            $nrorg              = $this->environment->getNrOrg();
            $cdoperador         = $this->environment->getCdOperador();
            $this->itensService->replicaAteracaoParaItemPai($nrseqorcamento, $nrorg, $cdoperador, $cdservorca);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Verirfica se o valor dos itens da proposta nã estão desatualizados 
     * de acordo com a tabela de preço ou a quantidade de medida do serviço relacionado
    **/
    public function validarValoresItensProp(Request\Row $request, Response $response){
        try{
            /*$row   = $request->getRow();
            $nrorg          = $this->environment->getNrOrg();
            $cdoperador     = $this->environment->getCdOperador();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $this->itensService->validarAtivacaoComposicao($nrseqorcamento, $nrorg);
            
            $validacao     = $this->itensService->validarValoresItensProp($nrseqorcamento, $nrorg, $cdoperador);
            $newVrProposta = ($validacao) ? $this->propostaService->atualizarValorProposta($nrseqorcamento, $nrorg) : false;
            $response->addDataSet(new DataSet("validarValoresItensProp", array("VRORCAMENTO" => $newVrProposta)));*/
            $response->addDataSet(new DataSet("validarValoresItensProp", array("VRORCAMENTO" => false)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se a proposta está ligada a composições inativas
     * Em caso afirmativo será retornada uma excessão.
    **/
    public function validarAtivacaoComposicao (Request\Row $request, Response $response){
        try{
            $row            = $request->getRow();
            $nrorg          = $this->environment->getNrOrg();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $acao           = isset($row["ACAO"]) ? $row["ACAO"] : "copiar";
            
            if($acao == "copiar"){ //copiar
                $this->itensService->validarAtivacaoComposicao($nrseqorcamento, $nrorg);
            } 
            
            $response->addDataSet(new DataSet("validarAtivacaoComposicao", array("POSSUICOMPINATIVA" => false)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Atualiza o nome contratual dos itens
     * 
     * @param Request\DataSet $request
    **/
    public function salvarNomeContratual(Request $request, Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $nrorg          = $this->environment->getNrOrg();
            $cdoperador     = $this->environment->getCdOperador();
            $this->itensService->salvarNomeContratual($rows, $nrorg, $cdoperador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function atualizarValorItensNegociacaoValores(Request\Row $request, Response $response) {
        try {
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $row = $request->getRow();
            $nrseqorcamento     = $row["NRSEQORCAMENTO"];
            $nritemorcamento    = $row["NRITEMORCAMENTO"];
            $nritemorcamentopai = $row["NRITEMORCAMENTOPAI"];
            $vrtotalitemorca    = $row["VRTOTALITEMORCA"];
            $this->itensService->atualizarValorItensNegociacaoValores($nrseqorcamento, $nritemorcamento, $vrtotalitemorca, $nritemorcamentopai, $nrorg, $cdoperador);
            
            $nrnegociacaoproposta = $this->itensService->getCodigoNegociacaoItemProposta($nritemorcamentopai, $nrorg);
            if ($nrnegociacaoproposta) {
                $this->propostaService->atualizarValoresNegociacaoAoEditarElemento($nrnegociacaoproposta, $nritemorcamentopai);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function verificarSeExisteDescontoNegociacaoProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaoproposta = $row["NRNEGOCIACAOPROPOSTA"];
            $nrorg = $this->environment->getNrOrg();

            $existeDesconto = $this->itensService->verificarSeExisteDescontoNegociacaoProposta($nrnegociacaoproposta, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("existe" => $existeDesconto)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificarSeExisteDescontoItensNegociacaoProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaoproposta = $row["NRNEGOCIACAOPROPOSTA"];
            $nrorg = $this->environment->getNrOrg();

            $existeDesconto = $this->itensService->verificarSeExisteDescontoItensNegociacaoProposta($nrnegociacaoproposta, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("existe" => $existeDesconto)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getServicoPercentualMinimoDescontoPermitidoNegociacaoProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaoproposta = $row["NRNEGOCIACAOPROPOSTA"];
            $cdoperador = $this->environment->getCdOperador();
            $nrorg = $this->environment->getNrOrg();

            $servico = $this->itensService->getServicoPercentualMinimoDescontoPermitidoNegociacaoProposta($nrnegociacaoproposta, $cdoperador, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("servico" => $servico)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * retorna o valor do serviço na tabela de preço 
    **/
    public function getValorUnitarioTblPrecoItemProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $cdservorca      = $row["CDSERVORCA"];
            $cdmedidavar     = $row["CDTPMEDIDAVARIAVEL"];
            $qtdmedida       = $row["QTMEDIDASERV"];
            $nrmoeda         = isset($row["NRMOEDA"]) ? $row["NRMOEDA"] : null;
            $nrorg           = $this->environment->getNrOrg();

            $vrpreco = $this->itensService->getValorUnitarioTblPrecoItemProposta($nrtipoorcamento, $cdservorca, $cdmedidavar, $qtdmedida, $nrorg, $nrmoeda);
            $response->addDataSet(new DataSet("fct_itemorcamento_widget", array("VRPRECO" => $vrpreco)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Atualiza o valor do rateio de um item
    **/
    public function atualizarValorRateioItens(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            
            $nrseqorcamento       = $row["NRSEQORCAMENTO"];
            $nritemorcamento      = $row["NRITEMORCAMENTO"];
            $nrnegociacaoproposta = $row["NRNEGOCIACAOPROPOSTA"];
            $this->itensService->atualizarValorRateioItens($nrseqorcamento, $nritemorcamento, $nrnegociacaoproposta, $nrorg, $cdoperador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getMedidaVariavelEscopo(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrorg();
            
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $cdservorca      = $row["CDSERVORCA"];

            $medida = $this->itensService->getMedidaVariavelEscopo($nrtipoorcamento, $cdservorca, $nrorg);
            if ($medida) {
                $response->addDataSet(new DataSet("retorno", $medida));
            }
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validarNegociacaoValorDescontoOperadorEPefil(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrorg      = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            $nrNegociacaoProposta = isset($row["NRNEGOCIACAOPROPOSTA"]) ? $row["NRNEGOCIACAOPROPOSTA"] : $row["NRNEGOCIACAOCONTRATO"];
            $resposta = $this->itensService->validarNegociacaovalorDescontoOperadorEPerfil($nrNegociacaoProposta, $nrorg, $cdoperador);
            $response->addDataSet(new DataSet("request",array("resulted" => $resposta)));
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validarNegociacaoValorDescontoOperadorEPefilContrato(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrorg      = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            $nrNegociacaoContrato = $row["NRNEGOCIACAOCONTRATO"];
            $resposta = $this->itensService->validarNegociacaovalorDescontoOperadorEPerfilContrato($nrNegociacaoContrato, $nrorg, $cdoperador);
            $response->addDataSet(new DataSet("request",array("resulted" => $resposta)));
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>