<?php

namespace Teknisa\SM\Service;

use Teknisa\Libs\Util\ConnectionCustom;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\IntegracaoCrmFactory;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Util\Procedure\CriaMnemonico;


class IntegracaoCrmService {
    
    protected $integracaocrmfactory;
    protected $newCode;
    protected $connectionCustom;
    protected $fileServerBaseUrl;
    protected $environment;
    protected $connection;
    protected $entityManager;
    
    public function __construct(ConnectionCustom $connectionCustom, Environment $environment, $fileServerBaseUrl, EntityManager $entityManager){
        $this->connectionCustom = $connectionCustom;
        $this->environment = $environment;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->entityManager = $entityManager;
    }
    
    /**
     * Executa a procedure CRIA_MNEMONICO.
     * 
     * @param integer $nrorg
     * @param string  $tipo
     * @param integer $nrseq
     * 
     * @return string
     */
    public function executarProcedureCriaMnemonico($nrorg, $tipo, $nrseq) {
        $procedure = new CriaMnemonico($this->connection);
        $mnemonico = $procedure->executar($nrorg, $tipo, $nrseq);
        return $mnemonico;
    }
    
    /**
     * Efetua as configurações necessárias para o usuário do CRM
     * salvar a proposta
     * @params array $rows dados da proposta recebidos do CRM
    **/
    public function integracaoCrm($rows){
        try{
            $nrorg = $rows["requisitante"]["nrorg"];
            $entityManagerConnection = $this->connectionCustom->updateVpdConnection($nrorg);
            
            if ($entityManagerConnection !== false) {
                $this->entityManager = $entityManagerConnection;
            }
            
            $this->connection = $this->entityManager->getConnection();
            $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
            $this->integracaocrmfactory = new IntegracaoCrmFactory($this->entityManager);
            
            $retorno = $this->salvaDadosPropostaOport($rows, $nrorg);
            
            return $retorno;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Salva a proposta com os dados recebidos do crm
     * @params array $rows dados da proposta recebidos do CRM
    **/
    public function salvaDadosPropostaOport($rows, $nrorg){
        try{
            $this->connection->beginTransaction();
            
            $nrSeqOrcamentoModelo = $rows["proposta"]["modelo"];
            $informacoesModelo = $this->getDadosModelo($nrorg, $nrSeqOrcamentoModelo);
            
            $nrseqOrcamento = $this->newCode->getCodeByTableName("FCT_ORCAMENTO", $nrorg, 1, 10, true);
            $this->insereProposta($nrorg, $nrseqOrcamento, $rows, $informacoesModelo);
            
            $this->executarProcedureCriaMnemonico($nrorg, "O", $nrseqOrcamento);
            
            $operador = $rows["requisitante"]["operador"];
            foreach ($rows["proposta"]["itens"] as $rowItem) {
                $nrItemOracamento = $this->newCode->getCodeByTableName("FCT_ITEMORCAMENTO", $nrorg, 1, 10, true);
                $this->insereItens($nrItemOracamento, $nrorg, $nrseqOrcamento, $rowItem, $informacoesModelo, $operador);
            }
            
            $this->atualizarValorProposta($nrseqOrcamento, $nrorg);
            
            $nrrelacescopo = $this->newCode->getCodeByTableName("FCT_RELACESCOPOPROPOSTA", $nrorg, 1, 10, true);
            $escopo = $rows["proposta"]["escopos"];
            $filial = $informacoesModelo[0]["CDFILIAL"];
            $this->insereEscopoProp($nrrelacescopo, $nrseqOrcamento, $nrorg, $filial, $escopo, $operador);
            
            $documentos = $this->getDocumentoProposta($nrorg, $rows["proposta"]["modelo"], $escopo);
            $qtdDocumentos = count($documentos);
            
            if($qtdDocumentos > 0){
                $qtdDoc = 0;
                
                foreach ($documentos as $documento) {
                    $nrseqorcadoc = $this->newCode->getCodeByTableName("FCT_ORCAMENTODOC", $nrorg, 1, 10, true);
                    
                    if($qtdDoc == 0){
                        $idpddoc = "S";
                    } else {
                        $idpddoc = "N";
                    }
                    
                    $this->insereDocumentosProp($nrseqorcadoc, $nrseqOrcamento, $idpddoc, $documento, $nrorg, $operador);
                    $this->anexarDocumentosProposta($documento, $nrseqOrcamento, $nrorg, $nrseqorcadoc);
                    $qtdDoc++;
                }
            }
            
            $this->connection->commit();
            return $nrseqOrcamento;
            
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }    
    }
    
    /**
     * Retorna os dados da proposta presentes no modelo
     * @params $nrorg nr organização
     * @params $nrSeqOrcamentoModelo nr do modelo
    **/
    public function getDadosModelo($nrorg, $nrSeqOrcamentoModelo){
        $paramsModelo = array(
            "NRORCAMENTOMODELO" => $nrSeqOrcamentoModelo,
            "NRORG"             => $nrorg
        );
        
        $dataSet = $this->integracaocrmfactory->buscaDadosModelo($paramsModelo);
        return $dataSet;
    }
    
    /**
     * Atualiza o valor total da proposta
     * @params $nrseqOrcamento nr orcamento
     * @params $nrorg nr da organização
    **/
    public function atualizarValorProposta($nrseqOrcamento, $nrorg){
        $paramsSoma = array(
            "NRSEQORCAMENTO" => $nrseqOrcamento,
            "NRORG"          => $nrorg
        );
        $vrorcamento = $this->integracaocrmfactory->getSomaValorItensProposta($paramsSoma);
        
        $paramsAtualizaProp = array(
            "NRSEQORCAMENTO" => $nrseqOrcamento,
            "VRORCAMENTO"    => $vrorcamento[0]["VRTOTALITEMORCA"],
            "NRORG"          => $nrorg
        );
        $this->integracaocrmfactory->atualizarValorProposta($paramsAtualizaProp);
    }

    /**
     * Retorna os documentos do escopo
     * @params $nrSeqOrcamentoModelo nr do modelo
     * @params $nrorg nr da organização
    **/
    public function getDocumentoProposta($nrorg, $nrSeqOrcamentoModelo, $escopo){
        $paramsEscopo = array(
            "NRORCAMENTOMODELO" => $nrSeqOrcamentoModelo,
            "NRTIPOORCAMENTO"   => $escopo,
            "NRORG"             => $nrorg
        );
        
        $dataSet = $this->integracaocrmfactory->getDocumentoProposta($paramsEscopo);
        return $dataSet;
    }
    
    /**
     * Retorna o nome e a unidades do produto
     * @params $cdproduto código do produto
    **/
    public function getNomeProduto($cdproduto){
        $params = array(
            "CDPRODUTO" => $cdproduto
        );
        
        $dataSet = $this->integracaocrmfactory->getNomeProduto($params);
        return $dataSet;
    }
    
    /**
     * Retorna o nome do serviço
     * @params $cdservico código do serviço
    **/
    public function getNomeServico($cdservico){
        $params = array(
            "CDSERVORCA" => $cdservico
        );
        
        $dataSet = $this->integracaocrmfactory->getNomeServico($params);
        return $dataSet;
    }
    /**
     * Retorna o nome do parceiro de negócio
     * @params $cdservico código do parceiro de negocio
    **/
    public function getNomeParcNegocio($nrparcnegocio){
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio
        );
        
        $dataSet = $this->integracaocrmfactory->getNomeParcNegocio($params);
        return $dataSet;
    }
    /**
     * Retorna os dados do interlocutor
     * @params $cdservico código do interlocutor
    **/
    public function getDadosInterlocutor($nrinterlocutor){
        $params = array(
            "NRSEQINTERLOCUTOR" => $nrinterlocutor
        );
        
        $dataSet = $this->integracaocrmfactory->getDadosInterlocutor($params);
        return $dataSet;
    }

    /**
     * Anexa o documento da proposta e efetua um update na tabela FCT_ORCAMENTODOC 
     * @params $documento documento a ser anexado
     * @params $nrseqOrcamento orcamento
     * @params $nrorg nr organização
     * @params $nrseqorcadoc pk da tabela FCT_ORCAMENTODOC
    **/
    public function anexarDocumentosProposta($documento, $nrseqOrcamento, $nrorg, $nrseqorcadoc) {
        $nomeArquivo   = $documento["NMARQUIVO"];
        $tipo          = $documento["DSTIPO"];
        $tamanhoBytes  = $documento["VRTAMANHO"] * 1024;
        $urlFileServer = $documento["DSCAMINHOARQ"];
            
        $conteudo = @file_get_contents($this->fileServerBaseUrl . $urlFileServer);
        if ($conteudo !== false) {
            $file = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_proposta", $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
            $caminhoNovo = $file["path"];
                
            $params = array(
                "DSCAMINHOARQ"   => $caminhoNovo,
                "NRSEQORCADOC"   => $nrseqorcadoc,
                "NRSEQORCAMENTO" => $nrseqOrcamento,
                "NRORG"          => $nrorg
            );
                
            $this->integracaocrmfactory->atualizarCaminhoDocumentoProposta($params);
        }
    }
    
    /**
     * Insere os dados da proposta
     * @params $nrseqOrcamento pk da tabela
     * @params $nrorg nr organização
     * @params $rows dados da proposta
     * @params $informacoesModelo dados do item da tabela de modelo
    **/
    public function insereProposta($nrorg, $nrseqOrcamento, $rows, $informacoesModelo){
        if($rows["proposta"]["parceiro_negocio"] != null){
            $dadosParc = $this->getNomeParcNegocio($rows["proposta"]["parceiro_negocio"]);
            $nomeParc = $dadosParc[0]["NMPRINCIPALPARC"];
        } else {
            $nomeParc = null;
        }
        
        if($rows["proposta"]["interlocutor"] != null){
            $dadosInterloc = $this->getDadosInterlocutor($rows["proposta"]["interlocutor"]);
            $nomeInterloc     = $dadosInterloc[0]["EMAILINTERLOC"];
            $telefoneInterloc = $dadosInterloc[0]["NRTELEFONE1"];
            $emailInterloc    = $dadosInterloc[0]["NMINTERLOCUTOR"];
        } else {
            $nomeInterloc     = null;
            $telefoneInterloc = null;
            $emailInterloc    = null;
        }
        
        $dataValidade = date_create();
        $dataValidade = $dataValidade->modify("+" . $informacoesModelo[0]["QTDIASVALIDADE"] . " day");
        $dataValidade = $dataValidade->format("DD.MM.YYYY HH24:MI:SS");
        
        $paramsProposta = array(
            "NRORG"                 => $nrorg,
            "NRORGINCLUSAO"         => $nrorg,
            "NRSEQORCAMENTO"        => $nrseqOrcamento,
            "NRORCAMENTOMODELO"     => $rows["proposta"]["modelo"],
            "NRTIPOORCAMENTO"       => $rows["proposta"]["escopos"],
            "NRSEQINTERLOCUTOR"     => $rows["proposta"]["interlocutor"],
            "NRPARCNEGOCIO"         => $rows["proposta"]["parceiro_negocio"],
            "CDOPERRESPONSAVEL"     => $rows["proposta"]["operador_responsavel"],
            "NRSEQCONTATO"          => $rows["proposta"]["nrseqcontato"],
            "NRSEQOPORT"            => $rows["proposta"]["nrseqoport"],
            "CDOPERINCLUSAO"        => $rows["requisitante"]["operador"],
            "CDFILIAL"              => $informacoesModelo[0]["CDFILIAL"],
            "CDMODPGTO"             => $informacoesModelo[0]["CDMODPGTO"],
            "NRGRUPOFATURA"         => $informacoesModelo[0]["NRGRUPOFATURA"],
            "QTPARCELASFATU"        => $informacoesModelo[0]["QTPARCELASFATU"],
            "QTDIASCONTRATO"        => $informacoesModelo[0]["QTDIASCONTRATO"],
            "QTDIASVALIDADE"        => $informacoesModelo[0]["QTDIASVALIDADE"],
            "NRDIASCARENCIA"        => $informacoesModelo[0]["NRDIASCARENCIA"],
            "DSREFERENCIAORCA"      => $informacoesModelo[0]["DSREFERENCIAORCA"],
            "DSOBSERVACAOORCA"      => $informacoesModelo[0]["DSOBSERVACAOORCA"],
            "NRDIAVENCPARCELASFATU" => $informacoesModelo[0]["NRDIAVENCPARCELASFATU"],
            "NMPRINCIPALPARC"       => $nomeParc,
            "DSEMAIL"               => $emailInterloc,
            "NRTELEFONE"            => $telefoneInterloc,
            "DSINTERORCA"           => null,
            "IDFATURAORCA"          => "A",
            "IDATIVO"               => "S",
            "IDGERACONTRATO"        => "NOVO",
            "IDTRANSPORTADORA"      => "PROPRIA",
            "IDSITUACAOORCAMENTO"   => "ELABORACAO",
            "NRVERSAOORCAMENTO"     => 1,
            "NRORGULTATU"           => null,
            "NRORCAMENTO"           => null,
            "CDOPERULTATU"          => null,
            "DTULTATU"              => null,
            "CDFILICONTATO"         => null,
            "NRCONTATO"             => null,
            "NRPARCTRANSPORTADORA"  => null,
            "CDPRECBASESERV"        => null,
            "CDREDUCONT"            => null,
            "NRPARCINTERLOCUTOR"    => $nomeInterloc,
            "DTINICIOCONTRATO"      => null,
            "DTFIMCONTRATO"         => null,
            "NRSEQORCAMENTOPAI"     => null,
            "IDORCAFILHO"           => null,
            "NRORCAMENTORELAC"      => null
        );

        $this->integracaocrmfactory->insereProposta($paramsProposta);
    }
    
    /**
     * Insere os itens da proposta
     * @params $nrItemOracamento pk da tabela
     * @params $nrorg nr organização
     * @params $nrseqOrcamento nr proposta
     * @params $rowItem dados do item
     * @params $informacoesModelo dados do item da tabela de modelo
     * @params $operador operador logado
    **/
    public function insereItens($nrItemOracamento, $nrorg, $nrseqOrcamento, $rowItem, $informacoesModelo, $operador){
        $rowItem["servico"] = 15;
        $nomeItem = null;
        $unidade  = null;
        $idMultmedida = "N";
        $vritemsemdesc = $rowItem["valor_unitario"] * $rowItem["quantidade"] * $rowItem["quantidade_medida"];
        
        if($rowItem["servico"] != null){
            $dataSet  = $this->getNomeServico($rowItem["servico"]);
            $nomeItem = $dataSet[0]["NMSERVORCA"];
        } else {
            if($rowItem["produto"] != null){
                $dataSet  = $this->getNomeProduto($rowItem["produto"]);
                $nomeItem = $dataSet[0]["NMSERVORCA"];
                $unidade  = $dataSet[0]["SGUNIDADE"];
            }
        }

        if($rowItem["medida"] != null){
            $idMultmedida = "S";
        }
        
        $paramsItens = array(
            "VRPERCDESCORCA"        => $rowItem["ajuste"],
            "CDTPMEDIDASERV"        => $rowItem["medida"],
            "CDSERVORCA"            => $rowItem["servico"],
            "CDPRODUTO"             => $rowItem["produto"],
            "QTITEMORCAMENTO"       => $rowItem["quantidade"],
            "VRTOTALITEMORCA"       => $rowItem["valor_total"],
            "IDTPDESCORCA"          => $rowItem["tipo_ajuste"],
            "VRUNITITEMORCA"        => $rowItem["valor_unitario"],
            "QTMEDIDAORCA"          => $rowItem["quantidade_medida"],
            "NRGRUPOFATURA"         => $informacoesModelo[0]["NRGRUPOFATURA"],
            "CDMODPGTO"             => $informacoesModelo[0]["CDMODPGTO"],
            "NRDIAVENCPARCELASFATU" => $informacoesModelo[0]["NRDIAVENCPARCELASFATU"],
            "QTPARCELASFATU"        => $informacoesModelo[0]["QTPARCELASFATU"],
            "NRITEMORCAMENTO"       => $nrItemOracamento,
            "VRITEMSEMDESC"         => $vritemsemdesc,
            "NRORG"                 => $nrorg,
            "NRORGINCLUSAO"         => $nrorg,
            "SGUNIDADE"             => $unidade,
            "NMITEMORCAMENTO"       => $nomeItem,
            "CDOPERINCLUSAO"        => $operador,
            "IDMULTMEDVRTOT"        => $idMultmedida,
            "NRSEQORCAMENTO"        => $nrseqOrcamento,
            "IDTRANSPORTADORA"      => "PROPRIA",
            "IDPRECOLOCACAO"        => "PROPORCIONAL_TEMPO",
            "DSQTITEMORCA"          => "Referente ao Produto",
            "IDIMPRESSAOORCA"       => "S",
            "IDATIVO"               => "S",
            "IDITEMOBRIGORCA"       => "S",
            "IDFATURAORCA"          => "A",
            "IDSOMASUBITENS"        => "N",
            "IDSOMANOITEMPAI"       => "N",
            "IDSOMANOTOTALORCA"     => "N",
            "NRITEMORCAMENTOPAI"    => null,
            "IDADICCOBRITEMORC"     => "N",
            "IDADICITEMORC"         => "N",
            "IDBLOQALTORC"          => "N",
            "NRORGULTATU"           => null,
            "CDOPERULTATU"          => null,
            "DTULTATU"              => null,
            "DTINICFATORC"          => null,
            "NRTIPOHIERARQSERV"     => null
        );
        
        $this->integracaocrmfactory->insereItens($paramsItens);
    }
    
    /**
     * Insere os escopos da proposta 
     * @params $nrrelacescopo pk da tabela
     * @params $nrseqOrcamento orcamento
     * @params $escopo nr escopo
     * @params $nrorg nr organização
     * @params $filial nr filial
    **/
    public function insereEscopoProp($nrrelacescopo, $nrseqOrcamento, $nrorg, $filial, $escopo, $operador){
        $params = array(
            "NRORG"                 => $nrorg,
            "NRORGINCLUSAO"         => $nrorg,
            "NRTIPOORCAMENTO"       => $escopo,
            "CDFILIAL"              => $filial,
            "CDOPERINCLUSAO"        => $operador,
            "NRRELACESCOPOPROPOSTA" => $nrrelacescopo,
            "NRSEQORCAMENTO"        => $nrseqOrcamento,
            "IDATIVO"               => "S",
            "NRSEQMODDOCFCT"        => null,
            "NRORGULTATU"           => null,
            "CDOPERULTATU"          => null,
            "DTULTATU"              => null
        );
        
        $this->integracaocrmfactory->insereEscopoProp($params);
    }
    
    
    /**
     * Insere os documentos da proposta 
     * @params $nrseqorcadoc pk da tabela
     * @params $nrseqOrcamento orcamento
     * @params $documentos informações do documento
     * @params $nrorg nr organização
    **/
    public function insereDocumentosProp($nrseqorcadoc, $nrseqOrcamento, $idpddoc, $documentos, $nrorg, $operador){
        $params = array(
            "IDATIVO"             => "S",
            "IDIDIOMA"            => null,
            "TXMODELOORCADOC"     => null,
            "NRORG"               => $nrorg,
            "IDPADRAOORCADOC"     => $idpddoc,
            "NRSEQORCADOC"        => $nrseqorcadoc,
            "NRSEQORCAMENTO"      => $nrseqOrcamento,
            "DSTIPO"              => $documentos["DSTIPO"],
            "NMARQUIVO"           => $documentos["NMARQUIVO"],
            "VRTAMANHO"           => $documentos["VRTAMANHO"],
            "DSCAMINHOARQ"        => $documentos["DSCAMINHOARQ"],
            "NRSEQMODDOCFCT"      => $documentos["NRSEQMODDOCFCT"],
            "IDSITUACAOORCAMENTO" => $documentos["IDSITUACAOORCAMENTO"],
            "NRTIPOORCAMENTO"     => $documentos["NRTIPOORCAMENTO"],
            "CDOPERINCLUSAO"      => $operador,
            "NRORGINCLUSAO"       => $nrorg,
            "NRORGULTATU"         => null,
            "CDOPERULTATU"        => null,
            "DTULTATU"            => null
        );
        
        $this->integracaocrmfactory->insereDocumentosProp($params);
    }
}
