
// FILE: js/Util/Colors.js
function Colors() {

	this.TColorToHex = function(TColor) {
		var rgb = this.TColorToRGB(TColor);
		return this.rgbToHex(rgb);
	};

	this.TColorToRGB = function(TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return [r,g,b];
	};

	this.rgbToHex = function (rgb) {
	    return "#" + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1);
	};
	
	this.RGBToTColor = function(r, g, b){
		var Tcolor = r | (g << 8) | (b << 16);
		return Tcolor;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Colors', Colors);
});

// FILE: js/Util/Endereco.js
var Endereco = function (restEngine, ScreenService, MessageService, ZHPromise) {
    var self = this;

    //Busca pelo CEP na API dos correios e retorna o resultado por promise
    self.buscarCep = function(CEP) {
        CEP = CEP.replace(/[\D]+/g,'');

        if(CEP.length == 8) {
            return restEngine.post('/lib_findAddress', {
                    'requestType':'Row',
                    'row': {'CEP': CEP}
            });
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };

    /* Busca pelo CEP na API dos correios e trata o retorno.
     * Usar apenas de exemplo para fazer um método próprio!
     */
    self.validarCep = function(widget, CEP) {
        self.buscarCep(CEP).then(function(response) {
            var endereco = response.dataset.ENDERECO;

            if(endereco) {
                widget.currentRow.CEP = CEP;
                widget.currentRow.CDPAIS = (endereco.CDPAIS)? endereco.CDPAIS : '';
                widget.currentRow.NMPAIS = (endereco.NMPAIS)? endereco.NMPAIS : '';
                widget.currentRow.SGESTADO = (endereco.SGESTADO)? endereco.SGESTADO : '';
                widget.currentRow.NMESTADO = (endereco.NMESTADO)? endereco.NMESTADO : '';
                widget.currentRow.CDMUNICIPIO = (endereco.CDMUNICIPIO)? endereco.CDMUNICIPIO : '';
                widget.currentRow.NMMUNICIPIO = (endereco.NMMUNICIPIO)? endereco.NMMUNICIPIO : '';
                widget.currentRow.CDBAIRRO = (endereco.CDBAIRRO)? endereco.CDBAIRRO : '';
                widget.currentRow.NMBAIRRO = (endereco.NMBAIRRO)? endereco.NMBAIRRO : '';
                widget.currentRow.CDLOGRADOURO = (endereco.CDLOGRADOURO)? endereco.CDLOGRADOURO : '';
                widget.currentRow.DSLOGRADOURO = (endereco.DSLOGRADOURO)? endereco.DSLOGRADOURO : '';
                widget.currentRow.DSENDERECO = (endereco.DSENDERECO)? endereco.DSENDERECO : '';
            } else {
                widget.currentRow.CEP = '';
            }
        }, function(error) {            
            MessageService.getMessageByCode("PROXY_CONNECTION_FAIL").then(function(errorMessage) {
                var row = {
                    "CEP": CEP, 
                    "errorMessage": errorMessage
                };
                widget.currentRow.CEP = '';

                if(error.data)
                    errorMessage = error.data.error;
            
                ScreenService.showMessageByCode("REQUEST_CEP_ERROR", row, Constants.errorMessage);
            });
        });
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('Endereco', Endereco);
});

// FILE: js/Util/General.js
var General = function (ScreenService, templateManager, localStorageEngine, restEngine, maskEngine, ZhUtil, ZHPromise, ValidationEngine, metaDataFactory, MessageService) {
	var searchListDeleted;
	var parent = this;
	var cacheLoaded = false;

	//@TODO Criar objeto de mensagens para tradução.

	/*
	 * Função que atualiza o dataSourceFilter de um field que depende do resultado de
	 * outro. Realiza o reload apenas ao clicar no field, evitando requisições desnecessárias.
	 * OBS.: para o funcionamento correto, é necessário limpar o value do field dependente
	 * ao alterar o valor do field "pai".
	 *
	 * @param field dependentField
	 * @param array parameters
	 *
	 * Exemplo:
	 * General.onClickDependentField(meuField, {nomeParametro1: valorParametro1, nomeParametro2: valorParametro2});
	 */
	this.onClickDependentField = function (dependentField, parameters) {

		var dataSourceFilterNew = this.buildDataSourceFilter(angular.copy(dependentField.dataSourceFilter), parameters);

		var changedDataSourceFilter = compareDataSourceFilters(dependentField.selectWidget, dataSourceFilterNew, parameters);

		if (changedDataSourceFilter) {
			changeDependetFieldFilter(dependentField, dataSourceFilterNew);
		}

		if ((dependentField.selectWidget && changedDataSourceFilter) || (!dependentField.dataSource.data || !dependentField.dataSource.data.length) || searchListDeleted) {
			dependentField.reload();
		}
	};

	this.buildDataSourceFilter = function (dataSourceFilter, parameters) {
		if (!dataSourceFilter) {
			throw "DataSourceFilter is not defined!";
		}

		for (var position in dataSourceFilter) {
			if (parameters.hasOwnProperty(dataSourceFilter[position].name)) {
				dataSourceFilter[position].value = parameters[dataSourceFilter[position].name];
			}
		}

		return dataSourceFilter;
	};

	var changeDependetFieldFilter = function (dependentField, dataSourceFilterNew) {

		dependentField.dataSourceFilter = dataSourceFilterNew;
		dependentField.dataSource.lazyLoad = false;
		if (dependentField.selectWidget) {
			dependentField.selectWidget.dataSourceFilter = dataSourceFilterNew;
		}

		if (dependentField.selectWidget && dependentField.selectWidget.searchList) {

			searchListDeleted = true;
			delete dependentField.selectWidget.searchList;

		} else if (dependentField.selectWidget) {

			searchListDeleted = false;

		}

	};

	var compareDataSourceFilters = function (selectWidget, dataSourceFilterNew, parameters) {
		var changedDataSourceFilter = false;
		if (selectWidget) {
			var selectWidgetDataSourceFilter = selectWidget.dataSourceFilter;
			for (var position = 0; position < dataSourceFilterNew.length; position++) {
				if (parameters.hasOwnProperty(dataSourceFilterNew[position].name)) {
					if (selectWidgetDataSourceFilter[position].value != dataSourceFilterNew[position].value) {
						changedDataSourceFilter = true;
					}
				}
			}
		} else {
			changedDataSourceFilter = true;
		}
		return changedDataSourceFilter;
	};

	/*
	 * Função que gera e retorna a totalização e o nível para agrupamento dos pratos
	 * ou produtos correspondentes à máscara utilizada. Através de cada nível
	 * da máscara, é gerado o número correspondente à ele.
	 *
	 * @param string maskName
	 * @param int firstLevel
	 *
	 * @return array
	 *
	 * Exemplo:
	 * General.getLevelByMask('CDPICTPRAT', 1);
	 */
	this.getLevelByMask = function (maskName, firstLevel) {

		firstLevel = toNumber(firstLevel);

		var mask = getLocalVar(maskName);
		var splitMask = mask.split('.');
		var biggestLevel = splitMask.length;

		var level = [];
		var objLevel = {};

		for (var i = 0; i < biggestLevel; i++) {

			objLevel = {
				nivel: i + firstLevel,
				caracteres: i > 0 ? parseInt(level[i - 1].caracteres) + parseInt(splitMask[i].length) : parseInt(splitMask[i].length)
			};

			objLevel.nivel = objLevel.nivel.toString();
			objLevel.caracteres = objLevel.caracteres.toString();

			level.push(objLevel);
		}

		return level;
	};

	var toNumber = function (value) {

		if (!value && value !== 0) {

			throw 'firstLevel is not defined';

		} else if (typeof value !== 'number' && value) {

			value = parseInt(value);

			if (isNaN(parseInt(value))) {

				throw 'Invalid firstLevel';

			} else {

				return value;
			}
		}
		return value;
	};

	this.toggleReadOnlySelect = function (checkboxField, selectField) {

		if (checkboxField.checked) {
			selectField.readOnly = true;
			selectField.validations = {};
			selectField.setValue('');
		} else {
			selectField.readOnly = false;
			selectField.validations = {
				required: true
			};
		}
		templateManager.updateTemplate();
	};

	this.setLoggedUnit = function (widget) {
		widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
		widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
	};

	var currentIdgrupoper;
	var currentCallback;
	var currentParameters;
	var gruposDeOperador = {
		LCMR: 'A liberação de cardápio fora da meta dependente de autorização'
	};
	var fields = [
		{
			"name": "email",
			"label": "E-mail",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"template": "field/text-edit.html",
			"events": [],
			"fieldGroup": "AUTH",
			"validations": {
				"required": true,
				"mail": true
			}
        },
		{
			"name": "senha",
			"label": "Senha",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"fieldGroup": "AUTH",
			"template": "field/password-edit.html",
			"events": [],
			"validations": {
				"required": true
			}
        }
    ];
	var actions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.validateUser(args.owner.widget);"
                }
            ]
        }
    ];

	this.openAuthPopup = function (idgrupoper, message, callback, parameters) {

		if (!idgrupoper) {
			throw 'idgrupoper is not defined';
		}

		initAuthVariables(idgrupoper, callback, parameters);

		if (!message) {
			message = gruposDeOperador[currentIdgrupoper] || '';
		}

		var widgetAuth = metaDataFactory.widgetFactory({
			"id": 0,
			"name": "authWidget",
			"label": "Autenticação",
			"template": "widget/form.html",
			"container": templateManager.container,
			"isVisible": true,
			"popupNoBlock": false,
			"fieldGroups": [
				{
					"id": 100000,
					"name": "AUTH",
					"opened": true,
					"collapsible": false,
					"label": message
                }
            ],
			"fields": fields,
			"actions": actions,
			"events": []
		});

		ScreenService.openPopup(widgetAuth);

	};

	var initAuthVariables = function (idgrupoper, callback, parameters) {
		currentIdgrupoper = idgrupoper;
		currentCallback = callback || null;
		currentParameters = parameters || [];
	};

	this.validateUser = function (widget) {

		var self = this;

		if (widget.isValid()) {

			var email = widget.currentRow.email;
			var senha = widget.currentRow.senha;

			autenticar(email, senha).then(function (callback) {

				getPermission(callback.dataset.user.cdoperador).then(function (permission) {

					if (permission) {

						if (typeof currentCallback == 'function') {

							ScreenService.closePopup();

							currentCallback.apply(null, currentParameters);

						} else {

							throw 'callback is not a function';

						}

					} else {

						MessageService.showMessageByCode('lib#GEN_INVALID_OPERGROUP', null, { type: 'ERROR' } );

					}

				}, function (callback) {

					this.callbackError(callback);

				}.bind(self));

			}, function (callback) {

				this.callbackError(callback);

			}.bind(self));
		}

	};

	var autenticar = function (email, senha) {

		return restEngine.request('/Autenticar', {
			"requestType": "FilterData",
			"filter": [
				{
					name: "email",
					operator: "=",
					value: email
                },
				{
					name: "senha",
					operator: "=",
					value: senha
                }
            ]
		});

	};

	var getPermission = function (cdoperador) {

		return restEngine.request('/validateUserGroup', {
			"requestType": "Row",
			"row": {
				"idgrupoper": currentIdgrupoper,
				"cdoperador": cdoperador
			}
		}).then(function (callback) {

			return callback.dataset.permission.isValid;
		});

	};

	this.callbackError = function (callback) {

		if (callback.data) {

			ScreenService.showMessage(ScreenService.i18n(callback.data.error));

		} else {
			
			MessageService.showMessageByCode('lib#GEN_TIMEOUT', null, { type: 'ERROR' } );
		}
	};

	this.comparaPratoProduto = function (pinicial, pfinal, mascara) {
		var mask = {
			type: "fix",
			params: {
				mask: mascara
			}
		};
		pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
		pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
		var nvinicial = pinicial.length;
		var nvfinal = pfinal.length;
		var menornv;
		if (nvinicial > nvfinal) {
			menornv = nvfinal;
		} else {
			menornv = nvinicial;
		}

		var resultado = true;
		var i = 0;

		while (pinicial[i] == pfinal[i] && i < menornv) {
			i++;
		}

		if ((pinicial.hasOwnProperty(i) && !pfinal.hasOwnProperty(i)) || (pinicial[i] > pfinal[i])) {
			resultado = false;
		} else {
			resultado = true;
		}

		return resultado;
	};

	var reportWidget = {
		"id": 0,
		"name": "reportWidget",
		"label": "Tipo de Relatório",
		"template": "widget/form.html",
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};
	var reportFields = [
		{
			"id": 99999999,
			"name": "QR2TYPE",
			"label": "",
			"class": 12,
			"readOnly": false,
			"isVisible": true,
			"isVisibleForm": true,
			"isVisibleGrid": false,
			"template": "field/radio-group.html",
			"events": [],
			"valueField": "value",
			"descriptionField": "name",
			"defaultOption": 0,
			"dataSource": {
				"data": [
					{
						name: "pdf",
						value: "PDF"
					},
					{
						name: "xls",
						value: "XLS"
					},
					{
						name: "rtf",
						value: "RTF"
					},
					{
						name: "txt",
						value: "TXT"
					}
				]
			}
        }
    ];
	var reportActions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.generateReport(args.owner.widget, args);"
                }
            ]
        }
	];
	
	var reportCallback;
	var reportParameters;
	var reportWidgetParentName;

	this.openReportPopup = function (widget, callback, parameters, reportFormats) {

		reportCallback = callback;
		reportParameters = parameters || [];
		reportWidgetParentName = widget.name;
		reportWidget.container = widget.container;
		reportWidget.fields = reportFields;
		reportWidget.actions = reportActions;

		var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

		updateReportWidgetFormats(reportWidgetInitialized, reportFormats);

		ScreenService.openPopup(reportWidgetInitialized);
	};

	var updateReportWidgetFormats = function(reportWidget, reportFormats) {
		if (Array.isArray(reportFormats) && reportFormats.length) {
			var formatField = reportWidget.getField("QR2TYPE");
			formatField.dataSource.data = reportFormats;
		}
	};

	this.generateReport = function (widget, args) {

		var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

		reportWidgetParentCurrentRow.QR2TYPE = widget.currentRow.QR2TYPE;

		args.row = reportWidgetParentCurrentRow;

		if (typeof reportCallback == 'function') {

			reportCallback.apply(null, reportParameters);
		}

		ScreenService.closePopup();

	};

	this.saveRow = function (args, closeSwipe) {
		args.owner.widget.saveRow().then(function () {
			var currentRow = args.owner.widget.currentRow;
			currentRow.__is_new = false;
			currentRow.changed = false;
			args
				.owner
				.widget
				.reload();
			if (closeSwipe) {
				ScreenService.closeSwipe();
			}
		});
	};

	this.openSelectionOnce = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		if (!filterWidget.wasOpened) {
			filterWidget.edit();
			ScreenService.openPopup(filterWidget);
			filterWidget.wasOpened = true;
		}
	};

	this.openSelection = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		filterWidget.edit();
		ScreenService.openPopup(filterWidget);
	};

	this.getContainerName = function () {
		return templateManager.container.name;
	};

	this.backOrCancel = function (args) {
		var deferred = ZHPromise.defer();
		if (args.row.__is_new) {
			args.owner.widget.list(true);
			args.owner.widget.dataSource.data.pop();
			args.owner.widget.currentRow = {};
			deferred.resolve();
		} else if (args.owner.widget.mode === 'edit') {
			args.owner.widget.view();
			for (var i in args.owner.widget.backupRow) {
				args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
			}
			deferred.resolve();
			return deferred.promise;
		}
		ScreenService.goBack();
		return deferred.promise;
	};

	this.edit = function (widget) {
		widget.backupRow = Object.assign({}, widget.currentRow);
		widget.edit();
	};

	this.modEdit = function (widget) {
		parent.edit(widget);
		var widgetParent = widget.widgets;
		if (widgetParent.length)
			parent.setVisibleforTabbed(widgetParent, false, widget);
		// widget.template = widget.formTemplate = widget.templateToForm;
	};

	this.cancelModEdit = function (args, widgetParent) {
		parent.backOrCancel(args).then(function (success) {
			var widget = args.owner.widget;
			var widgetParent = widget.widgets;
			if (widgetParent.length)
				parent.setVisibleforTabbed(widgetParent, true);
			// widget.template = widget.formTemplate = widget.templateToView;
		});
	};

	this.setVisibleforTabbed = function (widgets, isVisible, exception) {
		widgets.forEach(function (widget) {
			if (exception) {
				if (exception.name != widget.name)
					widget.isVisible = isVisible;
			} else {
				widget.isVisible = isVisible;
			}
		});
	};

	this.setContextVar = function (name, containerName, value) {
		localStorageEngine.setLocalVar(containerName + '_' + name, value);
	};

	this.getContextVar = function (name, containerName) {
		return localStorageEngine.getLocalVar(containerName + '_' + name);
	};

	var _getWidget = function (widgetName) {
		var isPopup = arguments[1] || null;
		if (!!templateManager.container) {
			var widget;
			if (!isPopup) {
				widget = templateManager.container.getWidget(widgetName);
			} else {
				widget = templateManager.container.getWidget(widgetName);
				if(widget)
					if(widget.hasOwnProperty('__filterPopup')) 
						widget = widget.__filterPopup;
					else{
						widget = widget.appliedFilter;
					}
			}
			if (widget) {
				return widget;
			}
		}
		return null;
	};

	var _getField = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var field = widget.getField(name);
		if (field) {
			return field;
		}
		return null;
	};

	var _getAction = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var action = widget.getAction(name);
		if (action) {
			return action;
		}
		return null;
	};

	this.getFieldValue = function (name, widgetName) {
		var isPopup = arguments[2] || null;
		var widget = _getWidget(widgetName, isPopup);
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		} else {
			return "";
		}
	};

	this.setEnableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = false;
	};

	this.setDisableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = true;
	};

	this.setVisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = true;
	};

	this.setInvisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = false;
	};

	this.setEnableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = false;
	};

	this.setDisableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = true;
	};

	this.setVisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = true;
	};

	this.setInvisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = false;
	};

	this.now = function () {
		return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
	};

	this.today = function () {
		return moment(new Date()).format('DD/MM/YYYY');
	};

	var getCurrentModule = function() {
		var currentModule = window.location.hash.split('/').filter(function (_) {
			return _.length;
		}).pop();
		if (currentModule && currentModule.length > 0) {
			currentModule = currentModule.split(/_|#/)[0];
		}
		return currentModule;
	};

	this.openReport = function (route, reportName, params, isUrl) {
		isUrl = isUrl? true : false;  
		params.__report_name = reportName;
		
		var lastActiveWidget = templateManager.container.getActiveWidget().lastActiveWidget;
	
		if (lastActiveWidget && lastActiveWidget.beforeOpenReport) {
		    lastActiveWidget.beforeOpenReport({
		        reportParams: params
		    });
		}
		
		return restEngine.post(route, {
			"requestType": "Row",
			"row": params
		}, null, null, null, null, 3000000).then(function (response) {
			if (response && response.messages) {
				var report = response.messages.pop();
				var isLibOpenReport = report.message.includes('lib_openReport');
				if(isUrl || isLibOpenReport) {
					var url = report.message;
					if(isLibOpenReport) {
						var serviceUrl = window.serviceUrls.find(function(item) { 
							return item.package == getCurrentModule();
						}).baseUrl;
						if(!serviceUrl.includes('http')) {
							serviceUrl = window.location.origin + window.location.pathname.replace('//', '/') + serviceUrl;
						}
						var url = serviceUrl + url;
					}
					var reportOpened = window.open(url);
					if(!reportOpened) {
						MessageService.showMessageByCode('lib#GEN_POPUP_BLOCKER', null, { type: 'ERROR' } );
					}
				} else {
					if(params.QR2TYPE) {
						params.__FORMAT = params.QR2TYPE;
					}				
					
					var format = params.__FORMAT? params.__FORMAT: 'pdf';
					var data = report.message;
					if(data.includes('&format=')) {
						var splitedData = data.split('&format=');
						data = splitedData[0];
						format = splitedData[1];
					}
				    var a = document.createElement("a");
				    document.body.appendChild(a);
				    a.style = "display: none";
			        a.href = 'data:application/octet-stream;base64,'+data;
			        a.download = reportName+'.'+format;
			        a.click();
				}
			}
		}).catch(function (response) {
			if (response.data && response.data.error) {
				ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
			}
		});
	};

	// CRUD
	/**
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function (widget, afterValid) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.list(true);
					ScreenService.goBack();
					widget.reload().then(function () {
						deferred.resolve(success);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function (widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
					" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		} catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function (arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			} else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.deleteCrud = function (widget) {
		var deferred = ZHPromise.defer();
		MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
			widget.removeRow().then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					deferred.resolve();
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
				} else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
			});
		});
		return deferred.promise;
	};

    this.deleteCrudCheckedRows = function (widget) {
        var deferred = ZHPromise.defer();
        MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then(function (success) {
            widget.removeCheckedRows().then(function () {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode('lib#GEN_DELETE_BLOCK', null, { type: 'ERROR' } );
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

	this.justInteger = function (selector, retryCount) {
		if (!retryCount || retryCount > 5)
			retryCount = 5;
		if (!selector) {
			throw {
				"message": "Selector is not defined!"
			};
		}
		setTimeout(function () {
			var elem = $(selector);
			if (elem[0]) {
				var length = selector.split(',').length;
				for (var i = 0; i < length; i++) {
					if (elem.hasOwnProperty(i)) {
						elem[i].onkeypress = avoidNonInteger;
						elem[i].onpaste = preventEvent;
					}
				}
			} else {
				if (!retryCount) {
					console.error("Selector not found!");
				} else {
					retryCount--;
					this.justInteger(selector, retryCount);
				}
			}
		}.bind(this), 500);
	};

	function avoidNonInteger(e) {
		var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
		var numberCode = e.which;
		if (integerNumbers.indexOf(numberCode) < 0) {
			preventEvent(e);
		}
	}

	function preventEvent(e) {
		e.preventDefault();
	}

	function padLeft(value, length) {
		return (value.toString().length < length) ? padLeft("0" + value, length) : value;
	}

	function padRight(value, length) {
		return (value.toString().length < length) ? padRight(value + "0", length) : value;
	}

	this.getCurrentDate = function (format) {
		return moment().format(format);
	};

	this.setDateAttrToVarcharOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, currentRow[attrName].replace(':', ''));
	};

	this.setAttrOnCurrentRow = function (currentRow, attrName, value) {
		currentRow[attrName] = value;
	};

	this.setFloatAttrOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, this.valueToFloat(currentRow[attrName]));
	};

	this.valueToFloat = function (value) {
		if (typeof value == 'string' && !Util.isEmptyOrBlank(value)) {
			if (value.indexOf(',') == -1 && value.indexOf('.') == -1) {
				return parseFloat(value);
			} else if (value.indexOf(',') > value.indexOf('.')) {
				return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
			} else if (value.indexOf(',') < value.indexOf('.')) {
				return parseFloat(value.replace(/\,/g, ''));
			}
		} else {
			return value;
		}
	};

	this.toLocalDate = function (element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	this.toLocalDateFormat = function (value, params) {
		if (value) {
			var valueDate = moment(value, params.format);
			value = valueDate.format(params.toFormat);
		}

		return value;
	};

	maskEngine.registerMask('dateFormat', this.toLocalDate, this.toLocalDateFormat);

	var maskFunction = function (element, viewValue, params, callBack) {
		// do something
	};

	var maskFormatFunction = function (value, params) {
		if (value) {
			var length = value.length;
			if (length === 11)
				value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
			else if (length === 14)
				value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
			return value;
		}
	};

	maskEngine.registerMask('CPFCNPJ', maskFunction, maskFormatFunction);

	/*
	 * Função que verifica quais widgets serão exibidas baseado em quais projetos ela pode ser exibida.
	 * A função deve ser executada no 'ContainerAfterInit' mas deve ser estudado a possibilidade de ser executado
	 * no 'EventAgregatorOnOpenContainer' para que assim essa configuração possa ser aplicada em multiplas telas.
	 */

	var constAllWidgets;

	this.setVisibilityByProjectID = function (widgets) {
		if (widgets.length > 1) {
			widgets.forEach(function (childWidget) {
				parent.setVisibilityByProjectID(childWidget.widgets);
				if (childWidget.projects) {
					childWidget.projects.forEach(function (proj) {
						childWidget.isVisible = proj.projectID == templateManager.project.id || proj.projectID == "allProjects" ? true : false;
					});
				}
			});

		}

	};

	this.validateFilterPopup = function (args) {
		if(args.owner.hasOwnProperty('__filterPopup')){
			var filterPopup = args.owner.__filterPopup;
		
		if (!filterPopup.isValid())
			args.e.preventDefault();
		}	
	};

	var timeFormats = {
		"hh": isValidHour,
		"mm": isValidMinutesOrSeconds,
		"ss": isValidMinutesOrSeconds,
		"hh:mm": isValidHourAndMinutes,
		"mm:ss": isValidMinutesAndSeconds,
		"hh:mm:ss": isValidTime,
	};

	function isValidHour(hour) {
		if (parseInt(hour) > 23 || parseInt(hour) < 0)
			return false;
		else
			return true;
	}

	function isValidMinutesOrSeconds(minuteOrSecond) {
		if (parseInt(minuteOrSecond) > 59 || parseInt(minuteOrSecond) < 0)
			return false;
		else
			return true;
	}

	function isValidHourAndMinutes(hourAndMinute) {
		if (hourAndMinute) {
			var timeArray = hourAndMinute.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidMinutesAndSeconds(minutesAndSeconds) {
		if (minutesAndSeconds) {
			var timeArray = minutesAndSeconds.split(':');
			return isValidMinutesOrSeconds(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidTime(time) {
		if (time) {
			var timeArray = time.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]) && isValidMinutesOrSeconds(timeArray[2]);
		}
		return true;
	}

	var validTime = function (value, format) {
		var validFunction;
		if (!format)
			throw 'format is not defined';
		if (typeof format == 'string' && timeFormats.hasOwnProperty(format.toLowerCase()))
			validFunction = timeFormats[format.toLowerCase()];
		else
			throw 'time format is not registered';
		return {
			"valid": validFunction(value),
			"message": ScreenService.i18n("Não é um tempo válido.")
		};
	};

	this.registerTimeFormat = function (format, formatFunction) {
		if (typeof format !== 'string')
			throw 'time format must be an string';
		if (timeFormats.hasOwnProperty(format.toLowerCase()))
			throw 'time format already registered';
		timeFormats[format] = formatFunction;
	};

	ValidationEngine.registerValidation('isValidTime', validTime);

	/*
	 * Função que preenche um valor com um único caractere até um certo tamanho.
	 * Faz zerofill pela esquerda por default.
	 *
	 * @param string value
	 * @param int length
	 * @param boolean toRight
	 * @param char filler
	 *
	 * @return string
	 *
	 * Exemplos:
	 * General.fill('123', 10);            //'0000000123'
	 * General.fill('123', 10, true);      //'1230000000'
	 * General.fill('123', 10, null, '_'); //'_______123'
	 */
	this.fill = function (value, length, toRight, filler) {
		if (value) {
			value = value.toString();

			if (filler) {
				filler = filler.toString()[0];
			} else filler = '0';

			if (toRight)
				while (value.length + filler.length <= length) value = value + filler;
			else
				while (value.length + filler.length <= length) value = filler + value;
		}
		return value;
	};

	/*
	 * Função que faz a validação de CNPJ.
	 *
	 * @param string CNPJ
	 *
	 * @return boolean
	 */
	function validCNPJ(CNPJ) {
		if (CNPJ === undefined || CNPJ === null)
			return false;

		CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

		if (CNPJ.length != 14)
			return false;

		if (CNPJ == "00000000000000" ||
			CNPJ == "11111111111111" ||
			CNPJ == "22222222222222" ||
			CNPJ == "33333333333333" ||
			CNPJ == "44444444444444" ||
			CNPJ == "55555555555555" ||
			CNPJ == "66666666666666" ||
			CNPJ == "77777777777777" ||
			CNPJ == "88888888888888" ||
			CNPJ == "99999999999999")
			return false;

		var tamanho = CNPJ.length - 2;
		var numeros = CNPJ.substring(0, tamanho);
		var digitos = CNPJ.substring(tamanho);
		var soma = 0;
		var pos = tamanho - 7;

		for (var i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		var resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(0)) //debugger;
			return false; //'resultado' é o 1º digito de validação do CNPJ

		tamanho = tamanho + 1;
		numeros = CNPJ.substring(0, tamanho);
		soma = 0;
		pos = tamanho - 7;

		for (i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(1)) //debugger;
			return false; //'resultado' é o 2º digito de validação do CNPJ

		return true;
	}

	this.validateCNPJ = validCNPJ;

	/*
	 * Função que faz a validação de CPF.
	 *
	 * @param string CPF
	 *
	 * @return boolean
	 */
	function validCPF(CPF) {
		if (CPF === undefined || CPF === null)
			return false;

		CPF = CPF.toString().replace(/[\D]+/g, '');

		if (CPF.length != 11)
			return false;

		if (CPF == "00000000000" ||
			CPF == "11111111111" ||
			CPF == "22222222222" ||
			CPF == "33333333333" ||
			CPF == "44444444444" ||
			CPF == "55555555555" ||
			CPF == "66666666666" ||
			CPF == "77777777777" ||
			CPF == "88888888888" ||
			CPF == "99999999999")
			return false;

		var soma = 0;
		var resto;

		for (var i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(9, 10))) //debugger;
			return false; //'resto' é o 1º dígito de validação do CPF

		soma = 0;
		for (i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(10, 11))) //debugger;
			return false; //'resto' é o 2º dígito de validação do CPF

		return true;
	}

	this.validateCPF = validCPF;

	/*
	 * Função que faz a formatação de uma inscrição,
	 * seguindo o padrão de código genérico, CPF, CNPJ e
	 * inscrição genérica. Caso o parâmetro "force" for
	 * true, o método tenta forçar uma inscrição válida.
	 *
	 * Retorna um objeto com as seguintes propriedades:
	 *  * parameter //string de entrada contendo apenas números
	 *  * value     //string formatada conforme o tipo da inscrição
	 *  * tipo.cod  //char do código padrão do tipo da inscrição
	 *  * tipo.desc //string de descrição do tipo da inscrição
	 *  * valid     //boolean
	 *
	 * @param string inscricao
	 * @param boolean force
	 *
	 * @return object
	 */
	this.inscricaoFormat = function (inscricao, force) {
		if (inscricao === undefined || inscricao === null || inscricao === '')
			return {
				'parameter': '',
				'value': '',
				'tipo': {
					'cod': null,
					'desc': null
				},
				'valid': false
			};

		if (force === undefined || force === null)
			force = false;
		inscricao = inscricao.toString().replace(/[\D]+/g, '');

		var obj = {};
		obj.tipo = {};

		if (inscricao.length <= 5) {
			obj.tipo.cod = 'O';
			obj.tipo.desc = 'Código Genérico';
			obj.valid = true;
			obj.value = this.fill(inscricao, 5);
		} else {
			if (inscricao.length == 11) {
				obj.tipo.cod = 'F';
				obj.tipo.desc = 'Pessoa Física';
				obj.valid = this.validateCPF(inscricao);
				obj.value = inscricao.substr(0, 3) + '.' + inscricao.substr(3, 3) + '.' + inscricao.substr(6, 3) + '-' + inscricao.substr(9, 2);
			} else if (inscricao.length == 14) {
				obj.tipo.cod = 'J';
				obj.tipo.desc = 'Pessoa Jurídica';
				obj.valid = this.validateCNPJ(inscricao);
				obj.value = inscricao.substr(0, 2) + '.' + inscricao.substr(2, 3) + '.' + inscricao.substr(5, 3) + '/' + inscricao.substr(8, 4) + '-' + inscricao.substr(12, 2);
			} else force = true;

			if (force && obj.valid !== true) {
				obj.tipo.cod = 'I';
				obj.tipo.desc = 'Inscrição Internacional';

				if (inscricao.length <= 20) {
					obj.valid = true;
					obj.value = this.fill(inscricao, 20);
				} else {
					obj.valid = false;
					obj.value = inscricao.substr(0, 20);
				}
			}
		}

		obj.parameter = inscricao;
		return obj;
	};

	var CPFValidation = function (value, validationAttrs) {
		return {
			"valid": validCPF(value),
			"message": ScreenService.i18n('CPF Inválido')
		};
	};

	ValidationEngine.registerValidation('CPF', CPFValidation);

	var CNPJValidation = function (value, validationAttrs) {
		return {
			"valid": validCNPJ(value),
			"message": ScreenService.i18n('CNPJ Inválido')
		};
	};

	ValidationEngine.registerValidation('CNPJ', CNPJValidation);

	this.callRestEngine = function (dataSource, requestType, requestValue, disableLoader) {
		var defer = ZHPromise.defer();
		var params = {};
		
		switch (requestType) {
		case "Row":
			params = {
				"requestType": "Row",
				"row": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "DataSet":
			params = {
				"requestType": "DataSet",
				"dataset": requestValue,
				"disableLoader": disableLoader
			};
			break;

		case "FilterData":
			params = {
				"requestType": "FilterData",
				"filter": requestValue,
				"disableLoader": disableLoader
			};
			break;

		default:
			return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
		}

		return restEngine.request(dataSource, params);
	};

	/*
	 * Função que muda a acessibilidade de uma ou mais
	 * widgets.
	 *
	 * @param array(objects) widgets
	 * @param boolean enabled
	 *
	 * Exemplos:
	 * General.changeWidgetAcessibility(widget.widgets, false); //Desabilita o acesso às widgets filhas
	 * General.changeWidgetAcessibility(widget.widgets, true);  //Habilita o acesso às widgets filhas e faz algo mais depois
	 */
	this.changeWidgetAcessibility = function (widgets, enabled) {
		widgets.forEach(function (widget) {
			widget.enabled = enabled;
		});

		templateManager.updateTemplate();
	};

	/*
	 * Função que altera o template de um form
	 * entre edição e visualização. Retorna um boolean
	 * que indica se a widget era do tipo form e a
	 * alteração foi realizada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.formWidgetSwitchTemplate = function (widget) {
		if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

		if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
		if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

		if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
		else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

		templateManager.updateTemplate();

		return true;
	};

	/*
	 * Função que coloca uma widget em modo de edição,
	 * bloqueando as demais widgets. Retorna um boolean
	 * que indica se a alteração de modo foi executada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.widgetEdit = function (widget) {
		if (!widget.editing) {
			//Trata a edição de uma widget filha do tipo form
			if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
				var widgetsToDeactivate = widget.parent.widgets;

				this.changeWidgetAcessibility(
					widgetsToDeactivate.filter(function (subwidget) {
						return subwidget.name != widget.name;
					}).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
					false);
			} else //trata a edição dos demais casos
				this.changeWidgetAcessibility(widget.widgets, false);

			//propriedades a serem ignoradas na criação do backup
			var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

			//faz backup da currentRow para permitir a recuperação dos dados
			widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
			widget.edit();
			return true;
		} else return false;
	};

	/*
	 * Procedimento que cancela o formulário de uma
	 * widget, seja este de inserção ou de edição, e
	 * desbloqueia as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetCancelForm = function (widget) {
		//trata o cancelamento de uma widget filha do tipo form
		if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
			var widgets = widget.parent.widgets;

			this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
		} else //trata o cancelamento dos demais casos
			this.changeWidgetAcessibility(widget.widgets, true);

		//caso a widget estiver em modo de inserção, remove a row e fecha o swipe
		if (widget.currentRow.__is_new) {
			var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
			if (indexOfCurrentRow !== -1) {
				widget.dataSource.data.pop();
			}

			widget.list();

			ScreenService.closeSwipe();
		} else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
			Object.keys(widget.backupRow).forEach(function (key) {
				widget.currentRow[key] = widget.backupRow[key];
			}); //copia dados coluna por coluna para não perder referência do objeto da currentRow
			widget.view();
		}
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * própria widget. Provavelmente usado na action de
	 * 'voltar' do back_view.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToList = function (widget) {
		ScreenService.closeSwipe();
		widget.list();
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * widget parent. Provavelmente usado na action de
	 * 'voltar' do back_list.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToParent = function (widget) {
		ScreenService.closeSwipe().then(function () {
			widget.parent.list();
			widget.parent.activate();
		});
	};

	/*
	 * Procedimento que habilita inserção em uma widget,
	 * bloqueando as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetNewRow = function (widget) {
		this.changeWidgetAcessibility(widget.widgets, false);

		widget.newRow();
	};

	/*
	 * Função que salva os dados de uma widget e retorna
	 * para o modo adequado de visualização ou listagem.
	 * Faz exibição de mensagens de aviso para o usuário
	 * conforme padronização e desbloqueia as demais
	 * widgets.
	 *
	 * Basicamente, um saveCrud adaptado ao padrão.
	 *
	 * @param object widget
	 *
	 * @return promise
	 */
	this.widgetSave = function (widget) {
		var deferred = ZHPromise.defer();

		if (widget.isValid()) {
			widget.currentRow.changed = true;

			if (!widget.currentRow.__is_new)
				widget.currentRow.__is_new = false;
			else if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = widget.paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                	widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
			}

			widget.saveRow().then(function (response) {
				widget.currentRow.changed = false;
				delete widget.backupRow;

				if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
					widget.view();

					var widgets = widget.parent.widgets;
					this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
				} else {
					if (widget.currentRow.__is_new) {
						ScreenService.closeSwipe();
						widget.list();
					} else widget.view();

					this.changeWidgetAcessibility(widget.widgets, true);
				}

				if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
				else widget.reload();

				deferred.resolve(response);
			}.bind(this), function (error) {
				var errorMessage = 'Timeout';

				if (error.data) errorMessage = error.data.error;

				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else deferred.reject("Preencha todos os campos corretamente!");

		return deferred;
	};

	/*
	 * Função para validar containers abertos a partir de outros containers.
	 * Se o container atual não foi aberto a partir do container pai, a
	 * callback é executada. Se nenhuma callback for passada nos parâmetros,
	 * o container pai é aberto.
	 *
	 * @param: object {parentContainerName, callback, scope, params}
	 *
	 * Exemplos:
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato',callback: ScreenService.showMessage, scope: ScreenService, params:['Teste']});
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato'});
	 */
	this.checkParentContainer = function (containerProperties) {
		containers = templateManager.containers;
		if (!containers.hasOwnProperty(containerProperties.parentContainerName)) {
			if (angular.isFunction(containerProperties.callback)) {
				containerProperties.callback.apply(containerProperties.scope, containerProperties.params);
			} else {
				ScreenService.openWindow(containerProperties.parentContainerName);
			}
		}
	};

	var messagesWidget = {
		"id": 0,
		"name": "messagesWidget",
		"label": "Mensagens",
		"messagesDataset": "",
		"template": "widget/form.html",
		"floatingControl": {
			"customizationAction": {
				"filter": false,
				"view": false,
				"layout": false
			}
		},
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};

	var messagesFields = [{
		"label": "",
		"name": "CONTENT",
		"readOnly": true,
		"template": "field/textarea.html",
		"textRows": 14,
		"isVisible": true,
		"class": 12,
		"descriptionOrder": true,
		"showOnView": true,
		"showOnForm": true,
		"showOnList": true
    }];

	var messagesActions = [{
		"name": "cancelar",
		"label": "Cancelar",
		"idVisible": true,
		"hideIcon": true,
		"showAsAction": "back",
		"events": [{
			"name": "ActionEvent",
			"code": "ScreenService.closePopup();"
        }]
    }, {
		"name": "exportar",
		"label": "Exportar",
		"idVisible": true,
		"icon": "export",
		"showAsAction": "always",
		"events": [{
			"name": "ActionEvent",
			"code": "General.generateFileByDataSet(args.owner.widget.messagesDataset);"
        }]
    }];

	/* Função abre um popup com um Text Area a título de mostrar erros em lista.
	 * Recebe a widget de origem e um dataset com mensagens a serem exibidas.
	 * @params: widget, dataset, popupName
	 */
	this.showErrorMessages = function (widget, dataset, popupName) {
		messagesWidget.container = widget.container;
		messagesWidget.fields = messagesFields;
		messagesWidget.actions = messagesActions;

		var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
		var content = '';
		if (popupName !== undefined)
			messagesWidgetInitialized.label = popupName;
		messagesWidgetInitialized.messagesDataset = dataset;
		dataset.forEach(function (message) {
			content += message + '\n';
		});
		messagesWidgetInitialized.getField('CONTENT').setValue(content);
		ScreenService.openPopup(messagesWidgetInitialized);
	};

	/* Gera arquivo básico baseado em linhas de dataSet.
	 * @params: dataset, object {fileName}
	 * !!! fileExtension: por enquanto somente TXT;
	 * Exemplo:
	 * General.generateFileByDataSet(dataset, {fileExtension: 'txt', fileName: 'meuArquivo'});
	 */

	this.generateFileByDataSet = function (dataset, fileProperties) {
		var content = "";
		var name = 'content';
		var extension = 'text/plain';

		if (fileProperties !== undefined) {
			if (fileProperties.fileName !== undefined)
				name = fileProperties.fileName;
		}

		dataset.forEach(function (line) {
			content += line + "\r\n";
		});

		var element = document.createElement('a');
		element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
		element.setAttribute('download', name);

		element.style.display = 'none';
		document.body.appendChild(element);

		element.click();

		document.body.removeChild(element);
	};

	/**
	 * Este método verifica se a widget foi filtrada usando o zh-grid-filter
	 * @param Widget widget
	 * @return boolean
	 */
	this.isWidgetFiltered = function (widget) {
		var isFiltered = true;
		var currentFilter;
		if(widget.hasOwnProperty('__filterPopup'))
			currentFilter = widget.__filterPopup.currentFilter;
		else
			currentFilter = widget.appliedFilter.FILTER_ROW;
			
		var requiredFields = widget.fields.filter(function (field){
			return isRequiredFieldInFilter(field) || isRequiredFieldWithoutFilterProperties(field);
		});
		if(requiredFields.length && Array.isArray(currentFilter)) {
			isFiltered = false;
		}
		return isFiltered;
	};

	function isRequiredFieldInFilter(field) {
		return field.filterProperties && field.filterProperties.validations && field.filterProperties.validations.required;
	}

	function isRequiredFieldWithoutFilterProperties(field) {
		return field.isVisible && field.validations && field.validations.required && !field.filterProperties;
	}

	/**
	 * Este método valida a inserção de registros na widget
	 * @param Widget widget
	 */
	this.newRow = function (widget) {
		if(this.isWidgetFiltered(widget)) {
			widget.newRow();
		} else {
			MessageService.showMessageByCode('lib#GEN_NOT_FILTER', null, { type: 'ERROR' } );
		}
	};

    /*
     * Função que adiciona à currentRow o(s) valor(es)
     * da widget parent necessários para salvar o registro.
     *
     * @param object widget
     * @param object parents
     *
     * @return promise
     */

    this.saveWithParent = function(widget, parents) {
        if(Array.isArray(parents)) {
            parents.forEach(function(parent){
                widget.currentRow[parent] = widget.parent.currentRow[parent];
            });
            return this.widgetSave(widget);
        }
        else throw("\'Parents\' property should be an array, " + typeof parents + " given.");
    };
    
    /*
     * Aceita [- . , 0 1 2 3 4 5 6 7 8 9]
     *
     */
    
     this.justDigits = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonDigits;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justDigits(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonDigits(e) {
        var digits = [44, 45, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var digitCode = e.which;
        if (digits.indexOf(digitCode) < 0) {
            preventEvent(e);
        }
    }

    /**
     * Função responsável por chamar a maskCurrency sempre que o 
     * usuário desfocar do campo onde ela é aplicada.
     * Função deve ser usada no evento FieldOnBlur, junto com a 
     * removeMask, essa no FieldOnFocus.
     * 
     * @param {Field} field - field.
     * @param {Char} decimal - Caráctere que será usado para a casa decimal.
     * @param {Char} thousand - Caráctere que será usado para as unidades de milhar.
     * @param {Number} precision - Quantidade de casas decimais.
     */

    this.callMaskCurrency = function(field, decimal, thousands, precision) {
    	precision = precision || 2;
        var value = field.value();
        var params = {
            "decimal": decimal,
            "thousands": thousands,
            "precision": precision
        };
        field.value(this.maskCurrency(value, params));
    };

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

    /**
     * Função responsável por aplicar uma máscara do type currency
     * no field.
     * 
     * @param {String} value - value do field.
     * @param {Object} params - Objeto param da máscara com {decimal, thousand, precision}.
     */

    this.maskCurrency = function(value, params) {
        var thousands = Util.isEmptyOrBlank(params.thousands) ? params.thousand : params.thousands;
        if (!Util.isEmptyOrBlank(value)) {

            if (value[0] === '.' || value[0] === ',')
                value = '0' + value;
            if (params.decimal === ',')
                value = numberFormatBr(value);
            var newValue = "";
            value = value.split('.').join('');
            var dec = value.split(',')[1];
            var mil = [];
            var int = value.split(',')[0];

            if (dec === undefined) {
                dec = "";
                dec = dec.padEnd(params.precision, "0");
                int = value;
            } else if (dec.length < params.precision) {
                dec = dec.padEnd(params.precision, "0");
            } else if (dec.length > params.precision) {
                dec = parseFloat('1.' + dec).toFixed(params.precision).toString().split('.')[1];
            }

            var counter = angular.copy(int.length / 3);

            for (var i = 1; i <= counter; i++) {
                if (i != counter)
                    mil.push(thousands + int.substr(-3));
                else
                    mil.push(int.substr(-3));
                int = int.substr(0, (int.length - 3));
            }

            if (int != "") mil.push(int);

            mil = mil.reverse().join('');
            newValue = mil + params.decimal + dec;
            var regex = '^(^([0-9]{1,3}\\' + thousands + ')((\\d{3}\\' + thousands + '{1})|(\\d{3}\\' + params.decimal + '\\d{2,}))*$)|^(\\d{0,3}\\' + params.decimal + '\\d{2,})*$';
            regex = new RegExp(regex, 'g');
            if (regex.test(newValue))
                return newValue;
            else
                return 0;
        }
    };

    maskEngine.registerMask('maskCurrency', maskFunction, this.maskCurrency);
    
     /**
     * Função que retira os '.' das casas de milhar dos números
     * e troca as ',' dos decimais por '.'
     * Dessa forma o campo fica compativel com o BD do oracle.
     * 
     * @param {String} value - Valor numérico.
     */

    this.parseFloat = function(value) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(value)) {
            var loop = value.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        }
        if (/\,/g.test(value))
            value = value.toString().replace(',', '.');
        return parseFloat(value);
    };
    
    /**
     * Função que formata números float vindos do banco, ou em
     * padrão americano e passa para o formato brasileiro.
     * 
     * @param {String} value - Valor numérico.
     */
    
    var numberFormatBr = function(value) {
        if (/^(.*)(\,+).*(\.+)(.*)$/gm.test(value)) {
            var loop = value.toString().split(',');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace(',', '');
            }
        }
        if (/^\d{1,}\.{1}\d*$/g.test(value))
            value = value.toString().replace('.', ',');
		else if(/^(\d*)([\.]*)((\d|\.)*)$/.test(value)){
			value = value.split('.');
			value[value.length-1] = ','+value[value.length-1];
			value = value.join('');
		}
        return value.toString();
    };
    
    /**
     * Função criada para corrigir o erro da maxlength.
     * Em vez do maxlength deve ser usada a propriedade maxLengthCurrency,
     * que considera apenas os números inteiros como length.
     * 
     * Função deve ser usada no evento FieldOnKeydown.
     * 
     * @param {Args} args - objeto args.
     * @param {Field} field - field com a propriedade maxLengthCurrency.
     */
    
    this.lengthCurrency = function(args, field) {
        var value;
        var fieldValue;
        
        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = field.widget.currentRow[field.name + '_inline'];
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        } else {
            fieldValue = field.value();
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue.toString();
        }
    
        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        
        if (realLength >= field.maxLengthCurrency && !/^(\d+)(\,){1}(\d*)$/gm.test(value)) {
            var allowedKeys = [8, 37, 38, 39, 40, 46, 110, 188, 190, 194];
            if (!allowedKeys.includes(args.e.keyCode))
                args.e.preventDefault();
        }
    };
    
    var treatResponse = function(response, Authenticator) { //
		var url = response.responseURL;
	    var responseURL = null;
	    var responseData = response.response;
	    
	    if (url.indexOf('index.php/') > -1) {
	        responseURL = url.slice(url.indexOf('index.php/') + 9);
	    }
	    
		if(responseURL){
			var responseObj = JSON.parse(responseData);
	        var errorCode = responseObj.error;
			if(Authenticator){
				Authenticator.run(responseURL, responseObj, errorCode);
			} else if(errorCode && errorCode.indexOf('AUTH_') > -1){// Exception: AUTH_INVALID_UNIT 
	            var cleanError = parent.getCleanError(errorCode);
				MessageService.showMessageByCode(cleanError);
			}
		}
	};
	
	this.getCleanError = function(msg){
		return msg.match(/AUTH_[^\s]+/g)[0];
	};
	
	/**
	 * Overwrite XMLHttpRequest.prototype.open
	 * and add a listener on it.
	 */
	this.initRoutesListener = function(Authenticator) {
	    (function(open) {
	        XMLHttpRequest.prototype.open = function(method, url, async, user, pass) {
	            this.addEventListener("readystatechange", function() {
	            	try{
	                	treatResponse(this, Authenticator);
	            	} catch(e){
	            		return false;
	            	}
	            }, false);
	
	            open.call(this, method, url, async, user, pass);
	        };
	    })(XMLHttpRequest.prototype.open);
	};
	
	this.isValidFiliaisValuesCP = function(values){
		var arrFilialValida = getLocalVar('CPFILIALVALIDA');
        if(arrFilialValida && cacheLoaded){ 
        	var arrValues = [];
        	Array.isArray(values)? arrValues = values : arrValues.push(values);
        	var isInvalid = arrValues.some(function(value){return !arrFilialValida.includes(value)}.bind(this));
        	return !isInvalid;
        }else{
        	this.callRestEngine('lib#/lib_getValidOrganizationAndBranches', 'FilterData', []).then( function(result){
        		cacheLoaded = true; 
        		if(result.dataset){
        			var arrayToPush = [];
        			result.dataset.validBranches.forEach(function(row){arrayToPush.push(row.FILIAL)}.bind(this));
					setLocalVar('CPFILIALVALIDA', arrayToPush);
					this.isValidFiliaisValuesCP(values);
        		}
        	}.bind(this));
        }
	};
	
	this.validateFieldFiliaisCP = function(field){
		var arrFilialValida = getLocalVar('CPFILIALVALIDA');
		var filialValues;
        if(arrFilialValida && cacheLoaded){ 
            if(field.name == 'CDFILIAL')
                filialValues = field.value();
            else{
                if(field.outData.CDFILIAL){
                    filialValues = field.widget.currentRow[field.outData.CDFILIAL];
                }else{
                    return console.log('Não existe o campo CDFILIAL neste FIELD, usar a função validateFieldFiliaisCP(field)');   
                }
                if(filialValues && filialValues.length > 0){
                	var arrValues = [];
		        	Array.isArray(filialValues)? arrValues = filialValues : arrValues.push(filialValues);
		            var filtered = arrValues.filter(function(filial){return arrFilialValida.includes(filial)}.bind(this));
		            var invalids = arrValues.filter(function(filial){return !arrFilialValida.includes(filial)}.bind(this));
		            
		            if(filtered.length == 0 || !this.isValidFiliaisValuesCP(arrValues)){
		            	MessageService.showMessageByCode('lib#CP_BRANCH_NOT_ALLOWED', null, { type: 'ERROR' });
		            	var payload = {CDFILIAL:invalids};
		            	this.callRestEngine('lib#/lib_saveOrgFilial', 'DataSet', [payload]);
		            	/*
		            	Descomentar essa parte quando for para bloquear totalmente o uso de unidades nao cadastradas
		            	
		            	field.setValue('');
		            	field.widget.currentRow[field.name] = '';
			            //remove valores nao liberados do datasource.data
			            var filteredDataSet = field.dataSource.data.filter(function(dataSetRow){return arrFilialValida.some(function(row){return row == dataSetRow.CDFILIAL}.bind(this))}.bind(this));
			            field.dataSource.data = filteredDataSet;
			            
			            //Remove valores nao liberados do spam do autocomplete.
			            if(field.template == 'zh-select-autocomplete#field/select-autocomplete.html'){
				            var spamList = field.element.context.parentElement.getElementsByClassName('list-items')[0].children;
				            for (var i = 0; i < spamList.length -1; i++) {
				            	if(spamList[i].className != 'view-more' && !arrFilialValida.some(function(row){return spamList[i].getAttribute('data-item-identifier').indexOf(row) >= 0}.bind(this))){
							    	spamList[i].remove();
				            		i--
				            	}
							}
			            }//fim if
			            */
		            }
                }
            }
        }else{
        	this.callRestEngine('lib#/lib_getValidOrganizationAndBranches', 'FilterData', []).then(function(result){
        		cacheLoaded = true; 
        		if(result.dataset){
        			var arrayToPush = [];
        			result.dataset.validBranches.forEach(function(row){arrayToPush.push(row.FILIAL)}.bind(this));
					setLocalVar('CPFILIALVALIDA', arrayToPush);
					this.validateFieldFiliaisCP(field);
        		}
        	}.bind(this));
        }
	};
	
	this.setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            if(Array.isArray(outDataValue[key])){
                var newArray = [];
                outDataValue[key].forEach(function(value){ newArray[value] = value}.bind(this)); // jshint ignore:line
                this.setCurrentRowWithOutData(currentRow, valuesToSetOnCurrentRow, newArray);
            }
            else
                currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };
	
};

Configuration(function (ContextRegister) {
	ContextRegister.register('General', General);
});

// FILE: js/Util/LogDecorator.js
function LogDecorator(ZhuLogHelper) {
    /* Widget Getters */
    var getContentWidget = function(gridWidget) {
        return gridWidget.widgets[0].widgets[0];
    };

    /* Field Names */
    const ROUTE_FIELD_NAME              = 'ROUTE';
    const ROUTE_LABEL_FIELD_NAME        = 'ROUTE_LABEL';
    const TIME_FIELD_NAME               = 'TIMESTAMP';
    const USER_ID_FIELD_NAME            = 'USER_ID';
    const USER_NAME_FIELD_NAME          = 'USER_NAME';
    const PRODUCT_ID_FIELD_NAME         = 'PRODUCT_ID';
    const CONTENT_FIELD_NAME            = 'CONTENT';
    const HTTP_RESPONSE_CODE_FIELD_NAME = 'HTTP_RESPONSE_CODE';
    const REQUEST_TYPE_FIELD_NAME       = 'REQUEST_TYPE';
    const REQUEST_ID_FIELD_NAME         = 'REQ_ID';
    const TYPE_FIELD_NAME               = 'TYPE';
    const CONTAINER_FIELD_NAME_NAME     = 'CONTAINER_NAME';
    const WIDGET_NAME_FIELD_NAME        = 'WIDGET_NAME';
    const ORGANIZATION_ID_FIELD_NAME    = 'ORGANIZATION_ID';

    /* Field Labels */
    const USER_NAME_FIELD_LABEL    = 'User Name';
    const ROUTE_LABEL_FIELD_LABEL  = 'Action';

    /* Field Templates */
    const TEXT_EDIT_TEMPLATE       = 'field/text-edit.html';
    const SELECT_NATIVE_TEMPLATE   = 'field/select-native.html';
    const SELECT_MULTIPLE_TEMPLATE = 'field/select-multiple.html';
    const DATE_INTERVAL_TEMPLATE   = 'zh-interval-component#field/interval.html';
    const DATETIME_TEMPLATE        = 'field/datetime.html';

    var filterFieldsInAdvancedFieldGroup = [REQUEST_ID_FIELD_NAME, TYPE_FIELD_NAME, HTTP_RESPONSE_CODE_FIELD_NAME, CONTENT_FIELD_NAME];
    var fieldsNotUpdateTemplate          = [ROUTE_LABEL_FIELD_NAME];

    var updateWidgetFields = function (widget) {
        var routeField          = ZhuLogHelper.getField(widget, ROUTE_FIELD_NAME);
        var userNameField       = Util.clone(routeField);
        var routeLabelField     = Util.clone(routeField);

        userNameField.name     = USER_NAME_FIELD_NAME;
        userNameField.label    = USER_NAME_FIELD_LABEL;
        routeLabelField.name   = ROUTE_LABEL_FIELD_NAME;
        routeLabelField.label  = ROUTE_LABEL_FIELD_LABEL;

        ZhuLogHelper.moveField(widget, USER_ID_FIELD_NAME,        1);
        ZhuLogHelper.addField (widget, userNameField,             2);
        ZhuLogHelper.moveField(widget, TIME_FIELD_NAME,           3);
        ZhuLogHelper.moveField(widget, CONTAINER_FIELD_NAME_NAME, 4);
        ZhuLogHelper.moveField(widget, WIDGET_NAME_FIELD_NAME,    5);
        ZhuLogHelper.moveField(widget, ROUTE_FIELD_NAME,          6);
        ZhuLogHelper.addField (widget, routeLabelField,           7);

        ZhuLogHelper.editField(widget, ROUTE_LABEL_FIELD_NAME, {
            valueField: ROUTE_FIELD_NAME,
            descriptionField: ROUTE_LABEL_FIELD_NAME,
            template: SELECT_NATIVE_TEMPLATE,
            dataSource: {
                data: window.routesTranslate || []
            },
            filterProperties: {
                template: SELECT_MULTIPLE_TEMPLATE,
                operator: 'IN'
            },
            showOnList: true,
            maxWidth: '12%',
            class: 3
        });

        ZhuLogHelper.editField(widget, USER_ID_FIELD_NAME,        { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, USER_NAME_FIELD_NAME,      { showOnList: true, maxWidth: '15%', class: 4 });
        ZhuLogHelper.editField(widget, CONTAINER_FIELD_NAME_NAME, { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, WIDGET_NAME_FIELD_NAME,    { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TIME_FIELD_NAME,           { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, ROUTE_FIELD_NAME,          { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TYPE_FIELD_NAME,           { showOnList: true, maxWidth: '8%' , class: 3 });

        ZhuLogHelper.removeField(widget, PRODUCT_ID_FIELD_NAME);
        ZhuLogHelper.removeField(widget, ORGANIZATION_ID_FIELD_NAME);
    };

    var updateWidgetFieldsGridOnly = function(gridWidget) {
        gridWidget.fields.forEach(function(field) {
            if(field.name && !(filterFieldsInAdvancedFieldGroup.indexOf(field.name) != -1)) {
                field.fieldGroup = null;
            }
            if(field.filterProperties) {
                if(fieldsNotUpdateTemplate.indexOf(field.name) == -1) {
                    field.template = field.filterProperties.template;
                }
                if(field.filterProperties.template == TEXT_EDIT_TEMPLATE) {
                    field.filterProperties.operator = 'LIKE';
                }
            }
        });

        //General filter fields
        ZhuLogHelper.editField(gridWidget, TIME_FIELD_NAME, {
            template: DATETIME_TEMPLATE,
            dateFormat: "DD/MM/YYYY",
            timeFormat: "HH:mm:ss",
            displayFormat: "DD/MM/YYYY HH:mm:ss",
            "datetime-dateFormat": "dd/mm/yy",
            "datetime-timeFormat": "HH:ii:ss",
            filterProperties: {
                isVisible: true,
                showOnForm: true,
                label: 'Date',
                template: DATE_INTERVAL_TEMPLATE,
                dateFormat: "DD/MM/YYYY",
                displayFormat: "DD/MM/YYYY",
                intervals: [ {
                    "label": "Última semana",
                    "period": "last-week"
                }, {
                    "label": "Último mês",
                    "period": "last-month"
                } ]
            }
        });

        //Advanced filter fields
        ZhuLogHelper.editField(gridWidget, TYPE_FIELD_NAME, { class: 4 });
        ZhuLogHelper.editField(gridWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, REQUEST_ID_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, CONTENT_FIELD_NAME, {
            isVisible: false,
            fieldGroup: 'advancedFilter',
            template : TEXT_EDIT_TEMPLATE,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE',
                class    : 12
            }
        });
    };

    var updateWidgetFieldsFormOnly = function(formWidget) {
        ZhuLogHelper.editField(formWidget, HTTP_RESPONSE_CODE_FIELD_NAME, { class: 2 });
        ZhuLogHelper.editField(formWidget, REQUEST_TYPE_FIELD_NAME,       { class: 3 });
    };

    this.decorateWidget = function(gridWidget) {
        updateWidgetFields(gridWidget);
        updateWidgetFieldsGridOnly(gridWidget);

        var formWidget = getContentWidget(gridWidget);
        updateWidgetFields(formWidget);
        updateWidgetFieldsFormOnly(formWidget);

        gridWidget.currentRowDefaultFilter = {
            TIMESTAMP: {
                start : moment().add(-1, 'days').format('DD/MM/YYYY'),
                end   : moment().format('DD/MM/YYYY'),
                period: undefined
            }
        };
    };

    this.normalizeRows    = function(rows) { return rows; };
    this.normalizeFilters = function(filters) {};
}

Configuration(function(ContextRegister){
    ContextRegister.register('LogDecorator', LogDecorator);
});


// FILE: js/Util/MenuBuilder.js
var MenuBuilder = function (restEngine, MessageService, templateManager, metaDataFactory, eventAggregator, $window, requestDataSourceEngine, ApplicationContext, ScreenService) {
    var localProductMenu;
    var attrToReplace = [
        "label",
        "showMenu",
        "showHeader",
        "showDescriptionOrder",
        "onRequestIdleTimeout",
        "mainWindow",
        "logLevel",
        "aclEnable",
        "showMenuSearch",
        "userDataActions",
        "reportServiceUrl",
        "reportsPath",
        "inDeveloperMode",
        "showDevtoolsInSameWindow",
        "shouldBackToMainWindowOnReload",
        "shortcutKeys"
    ];

    var requestProductMenu = function () {
        var modulesPackage = $window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/containers.json").then(function (response) {
            var productObjMenu = response.data;
            var keys = Object.keys(productObjMenu);
            // get first project menu found
            return productObjMenu[keys[0]];
        });
    };

    var removeEmptyMenuGroups = function(groupMenus) {
        return groupMenus.filter(function(group) {
            return (group.menus && group.menus.length != 0) || group.name == "zhHistory";
        });
    };

    var orderMenus = function(menus) {
        menus.map(function(menu) {
            if(menu.menus) {
                menu.menus = orderMenus(menu.menus);
            }
            return menu;
        });

        return sortMenus(menus);
    };

    var sortMenus = function(menus) {
        return menus.sort(function(menuA, menuB) {
            if (menuA.order && menuB.order) {
                if (menuA.order < menuB.order) {
                    return -1;
                }
                if (menuA.order > menuB.order) {
                    return 1;
                }
            }
            if (!menuA.order && menuB.order) {
                return -1;
            }
            if (menuA.order && menuB.order) {
                return 1;
            }
            return 0;
        });
    };

    var prepareMenus = function (response) {
        var groupMenus = response.dataset.menu;
        window.routesTranslate = response.dataset.routesTranslate;
        var zhuLogWidget = templateManager.container ? templateManager.container.getWidget('zhuLog') : false;
        if(zhuLogWidget) {
            zhuLogWidget.getField('ROUTE_LABEL').dataSource.data = window.routesTranslate;
            $(window).trigger('resize.grid');
        }
        groupMenus = orderMenus(removeEmptyMenuGroups(groupMenus));
        var groupMenuHistory = [templateManager.project.getMenu("zhHistory")];
        templateManager.project.groupMenu = metaDataFactory.groupMenuFactory(groupMenus.concat(groupMenuHistory));

        //Force framework to apply ACL Rules
        var aux = {};
        Object.keys(templateManager.aclRules).forEach(function(ruleKey){
            aux[ruleKey] = templateManager.aclRules[ruleKey];
        });
        templateManager.aclRules = aux;
        //Force framework to apply ACL Rules

        eventAggregator.publish('menuBuilded');
        if(!localStorage.getItem('IS_LOGGED')) {
            MessageService.successNotificationByCode('lib#SUC_AUTHENTICATION');
            setTimeout(function() {
                if(!JSON.parse(localStorage.getItem('IS_SUPPORT_OPERATOR'))) {
                    var hasVisibleMenu = templateManager.project.groupMenu.find(function(menu) {
                        return menu.name != 'zhHistory' && (typeof menu.aclIsVisible === undefined || menu.aclIsVisible === true)
                    });
                    if(hasVisibleMenu) {
                        saveLoginData();
                        localStorage.setItem('IS_LOGGED', true);
                    } else {
                        MessageService.showCustomDialogByCode('lib#EMPTY_ACCESS_RULE_LIB', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function() {
                                    ApplicationContext.LibLoginController.logout();
                                }
                            } ]
                        });
                    }
                } else {
                    localStorage.setItem('IS_LOGGED', true);
                }
            }, 1500);
        }
    };

    var replaceProductAttrOnMenu = function (productMenu) {
        attrToReplace.forEach(function (attr) {
            templateManager.project[attr] = productMenu[attr];
        });
    };

    var setLocalProductMenu = function () {
        if (!$window.modulesPackage) {
            requestProductMenu().then(function (productMenu) {
                localProductMenu = productMenu;
            });
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    if(!window.notUseMicroservice) {
        eventAggregator.subscribe('loginSuccess', function (loginResponse) {
            if(!JSON.parse(localStorage.getItem('IS_LOGGED'))) {
                ApplicationContext.LibUtilities.menuFavoritos(true);
            }
            eventAggregator.publish('AboutOnClick');
            if (!$window.modulesPackage && !$window.keepTitle) {
                replaceProductAttrOnMenu(localProductMenu);
                templateManager.updateTitle(templateManager.project.label);
            }
            var filter = [{
                "name": "CONCATENATE_GROUP_MENU",
                "operator" : "=",
                "value" :  true
            }, {
                "name": "VISIBLE_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.VISIBLE_ELEMENTS || []
            }, {
                "name": "EXPIRED_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.EXPIRED_ELEMENTS || []
            }, {
                "name": "URL_ORIGIN",
                "operator" : "=",
                "value" :  window.location.origin
            }, {
                "name": "CURRENT_MODULE",
                "operator" : "=",
                "value" :  getCurrentModule()
            }, {
                "name": "CURRENT_LANGUAGE",
                "operator" : "=",
                "value" :  window.lang
            } ];
            requestDataSourceEngine.getDataSource('lib#/lib_buildMenu', filter)
                .then(prepareMenus)
                .catch(ApplicationContext.LibUtilities.showErrorMessage);
        });

        setLocalProductMenu();
    }

    var saveLoginData = function() {
        ApplicationContext.LibUtilities.saveCollectedData(localStorage.getItem('LOGIN_USER_ID'), 'LOGIN');
        ApplicationContext.LibUtilities.saveCollectedData(localStorage.getItem('LOGIN_PHP_VERSION_ID'), 'PHPVERSION');
        ApplicationContext.LibLoginService.setLogAccess();
        var licenseInfo = localStorage.getItem('LICENSE_INFO');
        if(licenseInfo) {
            ApplicationContext.LibLoginService.saveProductsPermission(licenseInfo);
            ApplicationContext.LibLoginService.saveVariableMeasure(licenseInfo);
        }
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('MenuBuilder', MenuBuilder);
});

// FILE: js/Util/Messages.js
function LibMessages(){
    this.ALE_EXPIRED_SESSION = 'Sua sessão expirou.<br/>Você será redirecionado para efetuar login novamente.';
    this.ALE_MIGRATION_IS_PENDENT = 'ATENÇÃO!<br/><br/>Esse produto foi atualizado recentemente, com necessidade de alterações na estrutura do banco de dados, sem que essas alterações tenham sido aplicadas.<br/>A utilização do produto nessa situação poderá acarretar erros.<br/><br/>Solicite ao administrador do sistema a regularização da alteração na estrutura do bando de dados.', 'INFORMATION';

    this.ERR_ATTEMPT_END = 'Tentativas esgotadas. Redirecionando...';
    this.ERR_ATTEMPT_PARTIAL = 'Senha incorreta.<br/>Tentativa %1/3';
    this.ERR_EXPIRED_SESSION_LOGIN = 'Sessão expirada. Redirecionando...';
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibMessages', LibMessages);
});


// FILE: js/Util/ReportDecorator.js
function ReportDecorator(eventAggregator, restEngine, templateManager, ZhLogger, ScreenService, eventEngine) {
    
    this.onContainerOpen = function() {
        try {
            var container = templateManager.container;
            validContainer(container);
            if (container.customReportsConfig) {
                var customReportsConfig = container.customReportsConfig;
                validCustomReportsConfig(customReportsConfig);
                var widget = selectWidget(container, customReportsConfig.widgetName);
                var containerName = container.customReportsConfig ? container.customReportsConfig.reportCodeName : container.name;
                
                var fieldToChangeData = selectField(widget, customReportsConfig.fieldName);
                
                if (customReportsConfig.reportIsTheMainFunction === undefined || customReportsConfig.reportIsTheMainFunction === true) {
                    buildData(containerName, fieldToChangeData);
                } else {
                    const separator = "', '";
                    const eventCode = "ReportDecorator.triggerBuildData('" + containerName + separator + customReportsConfig.widgetName + separator + customReportsConfig.fieldName + "')";
                    
                    addEvent({
                        entity: widget,
                        event: {
                            name: 'WidgetOnEnterOnce',
                            code: eventCode,
                            bind: true,
                        }
                    });
                }
                widget.beforeOpenReport = beforeOpenReport;
            }
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    eventAggregator.subscribe('onContainerOpen',this.onContainerOpen);
    
    this.triggerBuildData = function(containerName, widgetName, fieldName) {
        const container = templateManager.container;
        
        validContainer(container);
        
        const widget = selectWidget(container, widgetName);
        const fieldToChangeData = selectField(widget, fieldName);
        
        try {
            buildData(containerName, fieldToChangeData);
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    
    function buildData(containerName, fieldToChangeData) {
        const route = getServiceUrlBasedOnTriggerPackage() + '#/lib_getDataReportField';
        
        restEngine.post(route, {
            "requestType": "Row",
            "row": [containerName.substr(0, 6)]
        }).then(function (callback) {
            try {
                var data = callback.dataset.dataReport;
                
                handleErrors(data);
                
                const formattedData = getFormattedData(fieldToChangeData, data);
                
                formattedData.forEach(function(row) {
                    row.description = ScreenService.i18n(row.description);
                });
                
                setDataSource(fieldToChangeData, formattedData);
                applyDefaultValue(fieldToChangeData);
                
                /**
                 * Força o select do item para que haja alteração visual.
                 * Operações visuais como updateTemplate e redraw não surtiram efeitos.
                 */
                setTimeout(function() {
                    clickOnSelectedOption(fieldToChangeData.name);
                }, 500);
                
                addEvent({
                    entity: fieldToChangeData.getParent(),
                    event: {
                        name: 'WidgetOnEnter',
                        code: "ReportDecorator.onEnterReportWidget('" + fieldToChangeData.name + "')",
                        bind: true,
                    }
                });
            } catch (e) {
                ZhLogger.warning(e);
            }
        }.bind(this), function(error) {
            if (error.data && error.data.error) {
                throw error.data.error;
            } else {
                throw ScreenService.i18n('Ocorreu um erro ao buscar a lista de relátorios da tela.');
            }
        });
    }
    
    function getServiceUrlBasedOnTriggerPackage() {
        const fileName = templateManager.container.fileName;
        const aliasSplited = fileName.split('#').shift().split('_');
        
        if (aliasSplited.length) {
            const packageName = aliasSplited.shift();
            
            if (packageName) return packageName;
        }
        
        return 'lib';
    }
    
    function getFormattedData(fieldToChangeData, data) {
        return data
                .filter(function(reportElement) {
                    return !Array.isArray(reportElement);
                })
                .map(function(reportElement) {
                    if (Array.isArray(reportElement)) return;
                    
                    const newElement = {};
                    
                    newElement[fieldToChangeData.valueField] = reportElement.value;
                    newElement[fieldToChangeData.descriptionField] = reportElement.description;
                    newElement[fieldToChangeData.strategyField] = reportElement.strategy;
                    
                    return newElement;
                });
    }
    
    function handleErrors(data) {
        const errorIndex = data.findIndex(function(reportElement) {
            return Array.isArray(reportElement);
        });
        
        if (errorIndex === -1) return;
            
        if (data.length === 1) {
            const message = ScreenService.i18n('Ocorreu um erro ao buscar a lista de relatórios da tela.');
            
            ScreenService.errorNotification(message);
            
            throw message + ScreenService.i18n('Consulte a requisição correspondente para mais detalhes.');
        } else {
            const isQr2Parametrized = data[errorIndex].findIndex(function(error) {
                return error === ScreenService.i18n('Servidor de relatórios QR2 não está parametrizado.');
            }) === -1;
            
            const isBirtParametrized = data[errorIndex].findIndex(function(error) {
                return error === ScreenService.i18n('Servidor de relatórios BIRT não está parametrizado.');
            }) === -1;
            
            if (isQr2Parametrized || isBirtParametrized) {
                ScreenService.errorNotification(ScreenService.i18n('Alguns relatórios podem não estar disponíveis.'));
            }
            
        	ZhLogger.warning(ScreenService.i18n('Alguns relatórios podem não estar disponíveis. Consulte a requisição correspondente para mais detalhes.'));
        }
    }
    
    function setDataSource(fieldToChangeData, data) {
        if (!fieldToChangeData.dataSource.data || fieldToChangeData.dataSource.data.length == 0) {
            fieldToChangeData.dataSource.data = data;
        } else {
            const valueField = fieldToChangeData.valueField;
            const descriptionField = fieldToChangeData.descriptionField;
            const strategyField = fieldToChangeData.strategyField;
            
            const removingDuplicatedReports = function(report, index, reports) {
                return index === reports.findIndex(function(r) {
                    return r[valueField] === report[valueField] &&
                           r[descriptionField] === report[descriptionField] &&
                           r[strategyField] === report[strategyField];
                });
            };
            
            const newData = fieldToChangeData.dataSource.data.concat(data);
            
            fieldToChangeData.dataSource.data = newData.filter(removingDuplicatedReports);
        }
    }
    
    function applyDefaultValue(fieldToChangeData) {
        if (!fieldToChangeData.defaultValue || fieldToChangeData.defaultValue === 'firstRow') {
            const firstRow = fieldToChangeData.dataSource.getFirstRow();
            
            fieldToChangeData.defaultValue = firstRow[fieldToChangeData.valueField];
        }
        
        fieldToChangeData.applyDefaultValue();
    }
    
    function clickOnSelectedOption(fieldName) {
        const baseSelector = "#" + fieldName + " + div";
        const baseElement = document.querySelector(baseSelector);
        
        if (!baseElement) {
            return setTimeout(function recursiveCallClickOnSelectedOption() {
                clickOnSelectedOption(fieldName);
            }, 500);
        }
        
        document.querySelector(baseSelector).click();
        
        const selectedElementSelector = baseSelector + ' li[class*="selected"]';
        
        document.querySelector(selectedElementSelector).click();
    }
    
    function selectWidget(container, widgetName) {
        var widget = container.getWidget(widgetName);
        
        if (widget) {
            return widget;
        } else {
            throw ScreenService.i18n("A widget informada não existe dentro desse container.");
        }
    }
    
    function selectField(widget, fieldName) {
        var field = widget.getField(fieldName);
        
        if (field) {
            if (field.dataSource) {
                return field;
            } else {
                throw ScreenService.i18n("O field informado não contém a propriedade dataSource.");    
            }
        } else {
            throw ScreenService.i18n("O field informado não existe na widget informada.");
        }
    }
    
    function validContainer(container) {
        if (!container) {
            throw ScreenService.i18n("Container não existe.");
        }
    }
    
    function validCustomReportsConfig(config) {
        if (!config.widgetName) {
            throw ScreenService.i18n('A propriedade widgetName não existe dentro de customReportsConfig.');
        }
        
        if (!config.fieldName) {
            throw ScreenService.i18n('A propriedade fieldName não existe dentro de customReportsConfig.');
        }
    }
    
    function eventExistsByCode(events, searchedCode) {
        return events.filter(function(event){
            return event.code.indexOf(searchedCode) != -1;
        }).length > 0;
    }
    
    function eventExistsByName(events, searchedName) {
        return events.findIndex(function(event){
            return event.name === searchedName;
        }) !== -1;
    }
    
    function addCodeOnEvent(events, eventName, eventCode, placeOn) {
        if (!placeOn) placeOn = 'right';
        
        const eventIndex = events.findIndex(function(event) {
            return event.name === eventName;
        });
        const event = events[eventIndex];
        const endSemicolonIndex = event.code.lastIndexOf(';');
        const codeHasEndSemicolon = endSemicolonIndex !== -1 && endSemicolonIndex === event.code.length - 1;
        
        if (!codeHasEndSemicolon) {
            event.code += ';';
        }
        
        if (placeOn === 'right') {
            events[eventIndex].code += eventCode;
        } else {
            const newCode = eventCode + ';' + events[eventIndex].code;
            
            events[eventIndex].code = newCode;
        }
    }
    
    function addEvent(params) {
        const entity = params.entity;
        const event = params.event;
        
        const name = event.name;
        const code = event.code;
        const events = entity.events;
        
        if (eventExistsByName(events, name)) {
            if (!eventExistsByCode(events, code)) {
                addCodeOnEvent(events, name, code, event.placeOn);
            }
        } else {
            events.push({
                name: name,
                code: code,
            });
            
            if (event.bind) {
                eventEngine.bindEvent(entity, name, code);
            }
        }
    }
    
    function getSelectedOption(fieldName) {
        const selector = '#' + fieldName + ' + div li[class*="selected"]';
        
        return document.querySelector(selector);
    }
    
    function getCurrent(fieldName) {
        const selector = '#' + fieldName + ' + div > span';
        
        return document.querySelector(selector).textContent;
    }
    
    this.onEnterReportWidget = function(fieldName) {
        const field = templateManager.container.getActiveWidget().getField(fieldName);
        
        if (!field.value()) {
            applyDefaultValue(field);
        }
        
        if (!getSelectedOption(fieldName)) {
            setTimeout(function runClickOnSelectedOption() {
                const selectedOption = getSelectedOption(fieldName);
                
                if (!selectedOption) {
                    return setTimeout(runClickOnSelectedOption, 500);
                }
                
                const current = getCurrent(fieldName);
                
                if (!current) {
                    return clickOnSelectedOption(fieldName);
                }
            }, 500);
        }
    };
    
    function beforeOpenReport(params) {
        const reportParams = params.reportParams;
        const reportNameSplitted = reportParams.__report_name.split('.');
        const reportName = reportNameSplitted[0];
        const strategy = reportNameSplitted[1];
        
        reportParams.__report_name = reportName;
        reportParams.STRATEGY = strategy;
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('ReportDecorator', ReportDecorator);
});

// FILE: js/Util/ShortcutDecorator.js
function ShortcutDecorator(ApplicationContext, templateManager) {

    /**
     *  Default keys for CRUD icons
     */
    const defaultCRUDShortcutKeys = {
        "add": "mod+a",
        "edit": "mod+e",
        "delete": "mod+d"
    };

    /**
     * 
     * Get the 'mod' key name according to OS (Ctrl or Command)
     * @returns String(Ctrl, Command) / Boolean(false)
     */
    function getTooltipModKey() {
        var platform = window.navigator.platform,
            commandPlatforms = ['Macintosh', 'MacIntel', 'MacPPC', 'Mac68K'],
            ctrlPlatforms = ['Win32', 'Win64', 'Windows', 'WinCE'],
            mod = false;

        if (commandPlatforms.indexOf(platform) !== -1) {
            mod = 'Command + ';
        } else if (ctrlPlatforms.indexOf(platform) !== -1 || /Linux/.test(platform)) {
            mod = 'Ctrl + ';
        }
        return mod;
    }

    /**
     * 
     * Get shortcut key from shortcutKeys object of widget, if not defined, 
     * from project property and if not defined, 
     * from defaultCRUDShortcutKeys const in this function.
     * @param {any} widget 
     * @param {any} shortcut 
     * @returns String(Shortcut key) / Undefined
     */
    function getShortcutKey(widget, shortcut) {
        return (
            Util.getNestedProperty(widget, ['shortcutKeys', shortcut]) ||
            Util.getNestedProperty(templateManager, ['project', 'shortcutKeys', shortcut]) ||
            defaultCRUDShortcutKeys[shortcut]
        );
    }
/**
 * 
 * Iterate recursively in widgets of container applying shortcut key in actions. 
 * @param {any} widgets 
 */
function applyShortcutOnAction(widgets) {
        var modTooltip = getTooltipModKey();
        widgets.forEach(function (widget) {
            if (widget.actions) {
                widget.actions.forEach(function (action) {
                    if (action.icon === 'plus' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'add');
                    } else if (action.icon === 'pencil' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'edit');
                    } else if (action.icon === 'trash' && !action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, 'delete');
                    } else if (!action.skipShortcut && !action.shortcutKey) {
                        action.shortcutKey = getShortcutKey(widget, action.icon);
                    }
                    if (modTooltip && action.shortcutKey && !action.tooltip && templateManager.project.showShortcutKeyAsTooltip !== false) {
                        action.tooltip = action.shortcutKey.toUpperCase().replace('MOD+', modTooltip);
                    }
                });
            }
            if (widget.widgets && widget.widgets.length) {
                applyShortcutOnAction(widget.widgets);
            }
        });
    }

    /**
     * 
     * If project have the property enableCRUDShortcutKeys == true, applies the shortcut keys in actions
     * @param {any} containerName 
     * @param {any} container 
     * @returns Container(container)
     */
    this.apply = function (containerName, container) {
        if (templateManager.project.enableCRUDShortcutKeys == true) {
            applyShortcutOnAction(container.widgets);
        }
        return container;
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ShortcutDecorator', ShortcutDecorator);
    ContainerDecoratorProvider.registerProvider(new ShortcutDecorator(ApplicationContext, templateManager));
});


// FILE: js/Util/Utilities.js
function LibUtilities(ScreenService, eventAggregator, templateManager, requestDataSourceEngine, restEngine, MessageService, ZhuAclConfig, General, ZHPromise) {
    var self = this;
    this.authenticationContainer = 'login#authentication';
    this.expiredSessionMethod = null;

    var STYLESHEET_TYPE    = 'stylesheet';
    var addLinkTagInHead = function(href, type) {
        var lnk = document.createElement('link');
        lnk.setAttribute('rel', type );
        if(type == STYLESHEET_TYPE) {
            lnk.setAttribute('type', 'text/css');
        }
        lnk.setAttribute('href', href);
        document.getElementsByTagName('head').item(0).appendChild(lnk);
    };

    var href = 'bower_components/tek-libraries/css/favorites.min.css';
    addLinkTagInHead(href, STYLESHEET_TYPE);
        
    this.showErrorMessage = function(response) {
        return MessageService.getMessageByCode('lib#UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode('lib#' + messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    if(!message.includes('ALE_SESSION_WILL_BE_ENDED')) {
                        return ScreenService.showMessage(message, 'ERROR');
                    }
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.redirectToLogin = function() {
        if(window.openLoginContainerWithoutRedirect) {
            ScreenService.openWindow(self.authenticationContainer);
        } else {
            self.openWindow(self.authenticationContainer);
        }
    };

    eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
        var validateExpiredSessionRoute = 'lib_validateExpiredSession';
        var logoutRoute = 'lib_logout';
        var loginRoute  = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if(container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if(!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if(route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if(route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var hash = localStorage.getItem('LOGIN_HASH');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if(window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if(eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if (hash) {
                requestConfig.headers['OAuth-Hash'] = hash;
                requestConfig.headers['OAuth-Project'] = window.tekLoginProjectId;
            }
            if(route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for(var i = 0; i < widgets.length; i++) {
            if(widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if(self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [ {
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        } ];

        requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
            if(response && response.dataset && response.dataset.expiredSessionInfo) {
                var expiredSessionInfo = response.dataset.expiredSessionInfo;
                if(expiredSessionInfo.IS_EXPIRED) {
                    eventAggregator.publish('LibExpiredSession');
                } else {
                    if(self.expiredSessionMethod) {
                        clearTimeout(self.expiredSessionMethod);
                    }
                    Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                }
            } else {
                self.showErrorMessage(response);
            }
        }).catch(self.showErrorMessage);
    };

    this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    this.openMicroServiceUrl = function(args) {
        var containerName = args.owner.msContainerName;
        var currentModule, module;

        if(containerName.indexOf('#') == -1) {
            currentModule = module = '';
        } else {
            var subModule = containerName.split('#')[0];
            module        = subModule.split('_')[0];
            currentModule = getCurrentModule();
        }

        if (module && currentModule && (module != currentModule)) {
            var filter = [ {
                name: 'USER',
                value: localStorage.getItem('LOGIN_USER'),
                operator: '='
            }, {
                name: 'IS_LOGGED',
                value: localStorage.getItem('IS_LOGGED') || false,
                operator: '='
            }, {
                name: 'KEEP_CONNECTED',
                value: localStorage.getItem('LOGIN_KEEP_CONNECTED') || 'No',
                operator: '='
            }, {
                name: 'LOGIN_LANGUAGE',
                value: localStorage.getItem('LOGIN_LANGUAGE') || window.lang,
                operator: '='
            }, {
                name: 'LOGIN_SEPARATOR',
                value: localStorage.getItem('LOGIN_SEPARATOR'),
                operator: '='
            } ];

            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource(module + '#/lib_startSession', filter).then(function() {
                ScreenService.hideLoader();
                ScreenService.openLink(args.owner.url);
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    };

    this.openLinkByMenu = function(menu) {
        if(typeof menu === 'string') {
            menu = templateManager.project.getMenu(menu);
        }

        var args = { owner: {
            msContainerName: menu.msContainerName,
            url: menu.url
        } };
        self.openMicroServiceUrl(args);
    };

    this.openWindow = function(windowName, newWindow) {
        return requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    eventAggregator.subscribe("AboutOnClick", function () {
        if(!templateManager.versionData) {
            templateManager.versionData = true;
            var filter = [ {
                name: 'CURRENT_MODULE',
                operator: '=',
                value: getCurrentModule()
            } ];
            ScreenService.showLoader();
            requestDataSourceEngine.getDataSource('lib#/lib_getVersionInfo', filter).then(function (response) {
                ScreenService.hideLoader();
                templateManager.versionData = response.dataset.versionInfo;
                templateManager.versionData.dependencies.current = templateManager.versionData.dependencies.frontend;
                templateManager.versionData.dependencies.changeCurrentVersion = function(newVersion) {
                    templateManager.versionData.dependencies.current = templateManager.versionData.dependencies[newVersion];
                };

                if (!!templateManager.versionData.dependencies) {
                    localStorage.setItem('versionData', JSON.stringify(templateManager.versionData));
                } else {
                    localStorage.setItem('versionData', JSON.stringify(false));
                }
            }).catch(function(error) {
                ScreenService.hideLoader();
                self.showErrorMessage(error);
            });
        }
    });

    eventAggregator.subscribe('onContainerOpen', function(args) {
        var versionData = JSON.parse(localStorage.getItem('versionData'));

        if (!args.container.isNext && 
             versionData && versionData.dependencies &&
             templateManager.versionData && templateManager.versionData.dependencies
            ) {
            templateManager.versionData.name = versionData.name;
            templateManager.versionData.version = versionData.version;
            templateManager.versionData.dependencies.backend = versionData.dependencies.backend;
            templateManager.versionData.dependencies.frontend = versionData.dependencies.frontend;
            templateManager.versionData.dependencies.modules = versionData.dependencies.modules;
            templateManager.versionData.dependencies.changeCurrentVersion('frontend');
            templateManager.updateTemplate();
        }
        self.menuFavoritos(false);
    });
    
    this.menuFavoritos = function (forceExec) {        
        // Aguarda renderização do componente de Acesso Rápido
        setTimeout(function() {            
            var groupId   = localStorage.getItem('ACCESS_PROFILE');
            var projectId = window.projectId;
            
            var favoritesData = localStorage.getItem('LIB_FAVORITES_DATA');
            if(favoritesData !== null) {
                eventAggregator.publish('ZhFavoritesOnLoad', favoritesData);
            } else if(projectId && groupId && (localStorage.getItem('IS_LOGGED') || forceExec)) {
                var zhuAclFilter = ZhuAclConfig.getDefaultDataSourceFilter([groupId], projectId);
                // Busca o Controle de Acesso do Operador Logado
                requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', zhuAclFilter, 1, undefined, undefined, undefined, 120000, 120000, undefined, true).then(function(response) {
                    var dataset = response.dataset || {};
                    if(dataset) {                            
                        requestDataSourceEngine.getDataSource('lib#/lib_findFavorites', [], 1, undefined, undefined, undefined, 120000, 120000, undefined, true).then(function(result) {
                            var favorites = formatFavoritesData(result);                                          
                            if(favorites) {                                
                                // Verifica Controle de Acesso do Menu Favoritos
                                verificaAcessoFavoritos(dataset, favorites).then(function(favorites) {                        
                                    // Inclui Favoritos no componente de Acesso Rápido
                                    favorites = JSON.stringify(favorites);
                                    localStorage.setItem('LIB_FAVORITES_DATA', favorites);
                                    eventAggregator.publish('ZhFavoritesOnLoad', favorites);
                                }.bind(this));
                            }                            
                        }, function() {});
                    }
                });
            } else if(localStorage.getItem('IS_LOGGED') || forceExec) {
                // Rotina para usuários sem perfil quando o controle de acesso está desabilitado
                requestDataSourceEngine.getDataSource('lib#/lib_findFavorites', [], 1, undefined, undefined, undefined, 120000, 120000, undefined, true).then(function(result) {
                    var favorites = formatFavoritesData(result);
                    if(favorites) {                                                     
                        // Inclui Favoritos no componente de Acesso Rápido
                        favorites = JSON.stringify(favorites);
                        localStorage.setItem('LIB_FAVORITES_DATA', favorites);
                        eventAggregator.publish('ZhFavoritesOnLoad', favorites);
                    }                            
                }, function() {});
            }
        }, 2000);
    };

    function formatFavoritesData(result) {
        var favorites = result.dataset.favorites;
        return favorites[0].map(function(item) {
            return {
                id: item.ID,
                name: item.NAME,
                label: ScreenService.i18n(item.LABEL)
            };
        });
    }
    
    function verificaAcessoFavoritos(dataset, favorites){
        var deferred = ZHPromise.defer();
        var pos = null;
        // Captura ID's dos Menus Favoritos
        var menuFavoritesIds = favorites.map(function(a) {return a.id;});
        // Regra Padrão
        var defaultRule = dataset.zhu_access_control_default_rule.defaultRule;
        
        // Preenche Regra Padrão para todos os Favoritos
        favorites.forEach(function(favorite){
            // Teste de Inativação
            favorite.rule = defaultRule;
        });
        
        // Percorre o Controle de Acesso dos Menus
        dataset.zhu_access_control_rule_login.forEach(function(acl){
            // Identifica existência de Controle de Acesso para o Menu
            if(menuFavoritesIds.includes(acl.UNIQUE_ID)){
                pos = menuFavoritesIds.indexOf(acl.UNIQUE_ID);
                // Preenche Regra para o Menu
                favorites[pos].rule = acl.RULE;
            }
        });
            
        // Permanece somente com os Favoritos ATIVOS(A)
        favorites = favorites.filter(function(favorite) {
            return ((favorite.rule == 'A') || (favorite.rule == 'R') || localStorage.getItem('ACL_ADMINISTRATOR'));
        });
        
        deferred.resolve(favorites);
        return deferred.promise;
    };
    
    eventAggregator.subscribe("ZhFavoritesOnChange", function (args) {
        /** 
          *Adiciona/Deleta Menu Favoritos
          * args.fullList = array<{ id, name, label }>
          * args.changed = { item: { id, name, label }, status: 'add'|'delete' }
        */
        localStorage.setItem('LIB_FAVORITES_DATA', JSON.stringify(args.fullList));
        args.changed.changed = true;
        requestDataSourceEngine.setDataSource('lib#/lib_changeFavorites', [args.changed], undefined, undefined, undefined, undefined, true);
    });

    eventAggregator.subscribe("EventOnRequestError", function (eventData) {
        if(eventData.e && eventData.e.e && eventData.e.e.data && eventData.e.e.data.data) {
            var error = eventData.e.e.data.data;
            var config = eventData.e.e.data.config;
            //Expired or Invalid Token
            if(error.errorCode == 101 || error.errorCode == 100)  {
                eventAggregator.publish('LibInvalidToken', error.errorCode);
            } else if(error.errorCode == 102){
                eventAggregator.publish('LibInvalidTokenSupportOperator');
            } else if(error.errorCode == 20){
                eventAggregator.publish('LibConcurrentAccess');
            } else if(error.errorCode == 21 || error.errorCode == 22){
                eventAggregator.publish('LibAccessBlocked');
            }
        }
    });

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };

    /**
     * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
     * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
     * @param $productId      => Código do Produto (opcional).
     * @param $operatorCode   => Código do Operador (opcional).
     * @param $operatorName   => Nome do Operador (opcional).
     * @param $organizationId => Código do organização (opcional).
     */
    this.saveCollectedData = function(collectionIds, operationId, productId, operatorCode, operatorName, organizationId) {
        var row = {
            COLLECTION_IDS : collectionIds,
            OPERATION_ID   : operationId,
            PRODUCT_ID     : productId		|| null,
            OPERATOR_CODE  : operatorCode   || null,
            OPERATOR_NAME  : operatorName   || null,
            ORGANIZATION_ID: organizationId || null,
            changed        : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_saveCollectedData', [row], undefined, undefined, undefined, undefined, true);
    };

    this.addTranslateButton = function() {
        templateManager.projectPromise.then(function() {
            if(!window.hideTranslateButton) {
                var notification = {
                    "label": ScreenService.i18n("Atualizar Tradução"),
                    "tooltip": ScreenService.i18n("Atualizar Tradução"),
                    "name": "libTranslate",
                    "icon": "information",
                    "isVisible": true,
                    "events": [ {
                        "name": "NotificationEvent",
                        "code": "LibLoginController.updateTranslation();"
                    } ]
                };
                notification = Util.extend(new _Notification(), notification);
                templateManager.project.notifications = templateManager.project.notifications || [];
                templateManager.project.notifications.push(notification);
            }
        });    
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibUtilities', LibUtilities);
});


// FILE: js/Util/ViewFloatMask.js
var ViewFloatMask = function(maskEngine) {
    var keypressHandler = function (e, element, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var thousandsCode = opt.thousands.charCodeAt();
        var isThousands = (thousandsCode === e.which);
        var is_number = (e.which >= 48 && e.which <= 57);
        var is_dot = (e.which == 46);
        var is_comma = (e.which == 44);
        var is_minus = (e.which == 45);
        var hasSeparator = (element.val().indexOf(opt.decimal) != -1 || element.val().indexOf(opt.thousands) != -1);
        var hasNegative = (element.val().length > 0 || !opt.allowNegative);

        if ((hasSeparator && (is_dot || is_comma)) || (hasNegative && is_minus) || isThousands)
            e.preventDefault();

        if (is_dot || is_comma || is_number || is_minus)
            element.data('maskOptions', opt);
        else
            e.preventDefault();
    };

    var keyupHandler = function (e, element, params) {

        if(params.precision){
            var value = element[0].value;
            var decimals = value.substring(value.indexOf(params.decimal || ",")+1, value.length);
            if(value != decimals){
                if(decimals.length > params.precision){
                    element[0].value = value.substring(0, value.indexOf(params.decimal || ",")+(params.precision + 1));
                }

            }

        }

    };

    var viewFloat = function(element, viewValue, params, callBack) {
        var value = viewValue || '';
        element.val(value);
        params.callback = callBack;
        params.showRound = !('showRound' in params) ? true : params.showRound;
        params.showSymbol = false;

        element.on("keypress", function(event) {
            keypressHandler(event, element, params);
        });

        element.on("keyup", function(event) {
            keyupHandler(event, element, params);
        });

        element.on("blur", function() {
            element.text(unMaskViewFloat(element.val(), params));
            element.val(viewFloatFormat(element.val(), params));
            params.callback(element.text());
        });

        if (!!params.showRound) {
            element.on("focus", function() {
                var value = element.text();
                var viewValue = viewFloatFormat(element.val(), params);
                var currentValue = viewFloatFormat(element.text(), params);

                if (currentValue !== viewValue)
                    value = element.val();

                element.val(viewFloatFormat(value, params));
                element.select();
            });
        }
    };

    var viewFloatFormat = function(value, params) {
        var defaultOptions = Util.clone($.currency.i18n[$.currency.lang]);

        var valueFormated = "";
        try {
            var options = Util.extend(defaultOptions, (params || {}), true);
            valueFormated = $.currency.string(value, options);
        } catch (e) {
            valueFormated = value;
        }
        return valueFormated;
    };

    var unMaskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var fltValue = opt.allowZero && !opt.allowEmpty ? 0 : "";
        var valueFormated = value;

        if (!Util.isEmptyOrBlank(valueFormated)) {
            var decimal = opt.decimal;
            var thousands = opt.thousands;

            if (typeof valueFormated === "string") {
                if(opt.showSymbol)
                    valueFormated = valueFormated.replace(opt.symbol, '');

                if(thousands)
                    valueFormated = valueFormated.replace(new RegExp('\\' + thousands, 'g'), '');

                valueFormated = valueFormated.replace(new RegExp('\\' + decimal, 'g'), '.');
                valueFormated = removeInvalidChar(valueFormated);
            }
            fltValue = parseFloat(valueFormated);

            if ((!opt.allowZero && fltValue === 0) || $.currency.isInvalidValue(fltValue, opt))
                valueFormated = "";
        }
        return valueFormated;
    };

    var removeInvalidChar = function(value) {
        var index = value.indexOf('.');
        return index + 1 === value.length ? value.slice(0, index) : value;
    };

    var maskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);

        if (value)
            value = value.replace(new RegExp('\\' + '.', 'g'), opt.decimal);

        return value;
    };

    this.register = maskEngine.registerMask('viewFloat', viewFloat, viewFloatFormat);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewFloatMask', ViewFloatMask);
});

// FILE: js/Util/WidgetsMetaData.js
function WidgetsMetaData() {
    this.getExpiredSession = function() {
        return {
            "name": "EXPIRED_SESSION",
            "label": "Sessão Expirada",
            "isVisible": true,
            "showDescriptionOrder": false,
            "showAsModal": true,
            "template": "widget/form.html",
            "tekLibExpiredSession": true,
            "actions": [
                {
                    "name": "CONFIRM",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventConfirm(args)"
                        }
                    ]
                },
                {
                    "name": "CANCEL",
                    "label": "Sair",
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventCancel()"
                        }
                    ]
                }
            ],
            "events": [],
            "dataSource": [],
            "fields": [
                {
                    "name": "OPERATOR",
                    "label": "Usuário",
                    "template": "field/text-edit.html",
                    "class": 12,
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "PASSWORD",
                    "label": "Senha",
                    "template": "field/password.html",
                    "class": 12,
                    "focused": true,
                    "validations":{
                        required: true
                    },
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true,
                    "events":[
                        {
                            "name": "FieldOnKeyup",
                            "code": "LibLoginController.expiredSessionFieldOnKeyupPassword(args)"
                        }
                    ]
                }
            ]
        };
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('WidgetsMetaData', WidgetsMetaData);
});


// FILE: js/Service/Login.js
function LibLoginService(requestDataSourceEngine){
    this.startSession = function(user) {
        var filter = [ {
            name: 'USER',
            value: user,
            operator: '='
        }, {
            name: 'IS_LOGGED',
            value: localStorage.getItem('IS_LOGGED') || false,
            operator: '='
        }, {
            name: 'LOGIN_TOKEN',
            value: localStorage.getItem('LOGIN_TOKEN'),
            operator: '='
        }, {
            name: 'LOGIN_LANGUAGE',
            value: localStorage.getItem('LOGIN_LANGUAGE') || window.lang,
            operator: '='
        }, {
            name: 'LOGIN_SEPARATOR',
            value: localStorage.getItem('LOGIN_SEPARATOR'),
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_startSession', filter, 1, undefined, undefined, undefined, 120000)
            .then(function(data) {
                if(typeof data == 'string') {
                    return {
                        isError : true,
                        message : data
                    };
                }
                return data && data.dataset ? data.dataset.system : false;
            });
    };

    this.validateUser = function(cdOperator, password, isSupportOperator) {
        var filter = [ {
            name: 'USER',
            value: cdOperator,
            operator: '='
        }, {
            name: 'PASSWORD',
            value: password,
            operator: '='
        }, {
            name: 'IS_SUPPORT_OPERATOR',
            value: isSupportOperator,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('login#/validateUser', filter);
    };

    this.logout = function() {
        var filter = [ {
            name: 'PROJECT',
            value: window.tekLoginProjectId,
            operator: '='
        }, {
            name: 'ERASE_HASH',
            value: localStorage.getItem('LOGIN_ERASE_HASH'),
            operator: '='
        }, {
            name: 'HASH',
            value: localStorage.getItem('LOGIN_HASH'),
            operator: '='
        } ];
        return requestDataSourceEngine.getDataSource('lib#/lib_logout', filter, 1, undefined, undefined, undefined, 120000);
    };

    this.getModulesVersion = function(){
        return requestDataSourceEngine.getDataSource('lib#/lib_getModulesVersion', []);
    };

    this.generateModulesVersionFile = function() {
        return requestDataSourceEngine.setDataSource('lib#/lib_generateModulesVersionFile', [ { changed: true } ]);
    };

    this.downloadModulesVersionFile = function(fileName) {
        var serviceUrl = window.serviceUrl;
        window.open(serviceUrl + "/lib_downloadModulesVersionFile?requestType=DataSet&dataset[0][FILE_NAME]=" + fileName);
    };

    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var filter = [ {
            name: 'USER',
            value: user,
            operator: '='
        }, {
            name: 'ACCESS_PROFILE',
            value: accessProfile,
            operator: '='
        }, {
            name: 'OTHER_SESSION_VALUES',
            value: otherSessionValues,
            operator: '='
        }, {
            name: 'LOGIN_USER',
            value: localStorage.getItem('LOGIN_USER'),
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_updateSession', filter);
    };

    this.updateTranslation = function() {
        return requestDataSourceEngine.getDataSource('lib#/lib_updateTranslation', [], 1, undefined, undefined, undefined, 300000);
    };

    this.saveProductsPermission = function(licenseInfo) {
        var row = {
            LICENSE_INFO: licenseInfo,
            changed     : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_saveProductsPermission', [row], undefined, undefined, undefined, undefined, true);
    };

    this.saveVariableMeasure = function(licenseInfo) {
        var row = {
            LICENSE_INFO: licenseInfo,
            changed     : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_saveVariableMeasure', [row], undefined, undefined, undefined, undefined, true);
    };

    this.setLogAccess = function() {
        var row = {
            changed : true
        };

        return requestDataSourceEngine.setDataSource('lib#/lib_setLogAccess', [row], undefined, undefined, undefined, undefined, true);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginService', LibLoginService);
});



// FILE: js/Controller/Customization.js
function LibCustomizationController(templateManager, LibUtilities, ScreenService, MessageService) {
    var getCustomizationContainer = function() {
		return templateManager.container;;
	};

    var getCustomizationWidget = function() {
		return getCustomizationContainer().getWidget('customizationWidget');
	};

	this.customizationContainerAfterinit = function () {
        var route 	  = localStorage.getItem('LIB_CUSTOMIZATION_ROUTE');
        var label 	  = localStorage.getItem('LIB_CUSTOMIZATION_LABEL');
        var token 	  = localStorage.getItem('LOGIN_TOKEN');
        var hash 	  = localStorage.getItem('LOGIN_HASH') || '0';
        var user      = localStorage.getItem('LOGIN_USER');
        var languange = localStorage.getItem('LOGIN_LANGUAGE');
        var customURL = localStorage.getItem('LOGIN_CUSTOMIZATION_URL');
        
        // Clear angular session timeout to use only next session timeout.
        clearTimeout(LibUtilities.expiredSessionMethod);
        
        var url = customURL + route + "?header=0&menu=0" +
                                      "&token=" + token +
                                      "&hash=" + hash +
                                      "&user=" + user +
                                      "&languange=" + languange +
                                      "&productId=" + window.projectId +
                                      "&isZhAngular=1";                  
        getCustomizationWidget().sourceUrl = url;
        getCustomizationContainer().label  = label + ' ' + ScreenService.i18n('(Customização)');
    };

    this.customizationMenuOnClick = function(menu) {
        if(!menu.route) {
            MessageService.showMessageByCode('lib#ERR_NOT_FOUND_CUSTOM_ROUTE', null, { type: 'ERROR' });
        } else {
            localStorage.setItem('LIB_CUSTOMIZATION_ROUTE', menu.route);
            localStorage.setItem('LIB_CUSTOMIZATION_LABEL', menu.label);
            ScreenService.openWindow('lib#customizationWindow');
            if (ScreenService.isSideMenuOpen()) {
                ScreenService.toggleSideMenu();
            }
        }
    }
}
    
Configuration(function(ContextRegister){
    ContextRegister.register('LibCustomizationController', LibCustomizationController);
});

// FILE: js/Controller/Login.js
function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise, MessageService) {
    var self = this;
    var loginUserId, loginUserName, loginIsSupOper, loginSupOperId;
    window.tekLoginProjectId = angular.copy(window.projectId);

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var updateLanguage = function () {
        var langParam = Util.getUrlParameter('lang');
        var language = langParam || localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };

    var updateToken = function () {
        var authToken = Util.getUrlParameter('authToken');
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        return authToken;
    };

    var updateHashAndKeepConn = function () {
        var hash = Util.getUrlParameter('hash');
        if (hash) {
            localStorage.setItem('LOGIN_HASH', hash);
        }
        var keepConn = Util.getUrlParameter('keepConn');
        if (keepConn) {
            localStorage.setItem('LOGIN_KEEP_CONNECTED', keepConn);
        }
    };

    var updateSeparator = function() {
        var separator = Util.getUrlParameter('sep') || localStorage.getItem('LOGIN_SEPARATOR');
        if(separator) {
            localStorage.setItem('LOGIN_SEPARATOR', separator);
            if($.currency.i18n[window.lang]) {
                $.currency.i18n[window.lang].thousands = separator[0];
                $.currency.i18n[window.lang].inputThousands = separator[0];
                $.currency.i18n[window.lang].decimal = separator[1];
                $.currency.i18n[window.lang].inputDecimal = separator[1];
            }
        }
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        updateLanguage();
        updateSeparator();
        updateHashAndKeepConn();

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER');
            var useRedirect = localStorage.getItem('LOGIN_USE_REDIRECT');
            var authToken = updateToken();

            if((!user && !authToken) || useRedirect == 'S') {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };
                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                ScreenService.showLoader();
                LibLoginService.startSession(user).then(function (response) {
                    ScreenService.hideLoader();
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    ScreenService.hideLoader();
                    //Expired or Invalid Token
                    if(error.data && (error.data.errorCode == 101 || error.data.errorCode == 100)) {
                        self.logout();
                    } else if(error.data.errorCode == 20){
                        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
                        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function(){
                                    self.logout()
                                }
                            } ]
                        });
                    } else{
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                if(response.LICENSE_INFO) {
                    localStorage.setItem('LICENSE_INFO', response.LICENSE_INFO);
                }
                if(response.LOGIN_USER && response.LOGIN_TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                    localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);
                }
                
                localStorage.setItem('IS_SUPPORT_OPERATOR', response.IS_SUPPORT_OPERATOR);
                localStorage.setItem('LOGIN_USER_ID', response.USER_ID);
                localStorage.setItem('LOGIN_PHP_VERSION_ID', response.PHP_VERSION_ID);
                localStorage.setItem('ACL_ADMINISTRATOR', response.ACL_ADMINISTRATOR);
                localStorage.setItem('ACCESS_PROFILE', response.ACCESS_PROFILE);
                localStorage.setItem('LOGIN_CUSTOMIZATION_URL', response.CUSTOMIZATION_URL);

                if(response.IS_SUPPORT_OPERATOR) {
                    loginIsSupOper = response.IS_SUPPORT_OPERATOR;
                    loginSupOperId = response.SUPPORT_OPERATOR_ID;
                }

                loginUserId   = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('lib#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }
                
                if(response.VISIBLE_ELEMENTS) {
                    localStorage.setItem('LOGIN_VISIBLE_ELEMENTS', JSON.stringify(response.VISIBLE_ELEMENTS));
                }

                if(response.ACL_ADMINISTRATOR) {                    
                    LibUtilities.addTranslateButton();
                }

                defer.resolve(response);
            }
        }
    };

    this.logout = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('ACL_ADMINISTRATOR');
            localStorage.removeItem('ACCESS_PROFILE');
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LICENSE_INFO');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('IS_SUPPORT_OPERATOR');
            localStorage.removeItem('LOGIN_USER_ID');
            localStorage.removeItem('LOGIN_PHP_VERSION_ID');
            localStorage.removeItem('LIB_FAVORITES_DATA');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };
 
    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION', null, {
                type: 'INFORMATION',
                actions: [ {
                    label: ScreenService.i18n("Ok"),
                    code: self.logout
                } ]
            });
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    eventAggregator.subscribe("LibInvalidTokenSupportOperator", function (errorCode) {
        //Expired Token Support Operator
        MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION_SUPPORT_OPERATOR', null, {
            type: 'INFORMATION',
            actions: [{
                label: ScreenService.i18n("Ok"),
                code: self.logout
            }]
        });
    });

    eventAggregator.subscribe("LibConcurrentAccess", function () {
        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    eventAggregator.subscribe("LibAccessBlocked", function (errorCode) {
        MessageService.showCustomDialogByCode('lib#ACCESS_BLOCKED_BY_TIME_OR_DAY', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        MessageService.getMessageByCode('lib#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            var userId   = loginSupOperId || loginUserId;
            ScreenService.showLoader();
            LibLoginService.validateUser(userId, password, loginIsSupOper)
                .then(function() {
                    // Requisição getModulesVersion chamada para atualizar o tempo de sessão local (produto ou módulo)
                    LibLoginService.getModulesVersion();
                    ScreenService.hideLoader();
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function(error) {
                    ScreenService.hideLoader();
                    if(error.data && error.data.errorCode && error.data.errorCode == 20) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibConcurrentAccess');
                    } else if(error.data && error.data.errorCode && error.data.errorCode == 102) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibInvalidTokenSupportOperator');
                    } else if(error.data && error.data.errorCode && (error.data.errorCode == 21 || error.data.errorCode == 22)) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibAccessBlocked');
                    } else {
                        var params = {attempts: ++expiredSessionWidget.attempts};
                        MessageService.showMessageByCode('lib#ERR_ATTEMPT_PARTIAL', params, {type: 'ERROR'});
                        if (expiredSessionWidget.attempts >= 3) {
                            logoutExpiredSession();
                        }
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    };

    /**
     * @param user -> Object or null.
     *                {
     *                    NAME:  'User Name',
     *                    EMAIL: 'user.email@teknisa.com'
     *                }
     * @param accessProfile -> Array of objects or null. ID is the column NRPERFILACESSO from table PERFILACESSO.
     *                         [
     *                             {
     *                                 ID: '1',
     *                                 IDPERFILACESSOADM: 'S'
     *                             },
     *                             {
     *                                 ID: '2',
     *                                 IDPERFILACESSOADM: 'N'
     *                             }
     *                         ]
     * @param otherSessionValues -> Object or null. PS.: CDFILIALSELECIONADA not exists in session. Use CDFILIAL.
     *                              {
     *                                  CDFILIAL:  '9999',
     *                                  NRPARCNEGOCIO: '1',
     *                                  CDPICTPROD: 'a.a.aa.aaa'
     *                              }
     * @returns Promise
     */
    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var defer = ZHPromise.defer();
        ScreenService.showLoader();
        LibLoginService.updateSession(user, accessProfile, otherSessionValues).then(function(response) {
            ScreenService.hideLoader();
            if(response && response.dataset && response.dataset.sessionUpdated) {
                var sessionUpdated = response.dataset.sessionUpdated[0];
                localStorage.setItem('LOGIN_USER', sessionUpdated);
                defer.resolve();
            } else {
                defer.reject(response);
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            defer.reject(error);
        });

        return defer.promise;
    };

    this.expiredMenuOnClick = function() {
        MessageService.showMessageByCode('lib#ALE_EXPIRED_MENU', null, { type: 'ERROR' } );
    };

    this.updateTranslation = function() {
        MessageService.confirmMessageByCode('lib#ALE_UPDATE_TRANSLATION', null,
            {
                type: 'CONFIRMATION',
                success: function() {
                    ScreenService.showLoader();
                    LibLoginService.updateTranslation().then(function () {
                        var widget = templateManager.container.getActiveWidget();
                        if(self.checkContainerIsZhNext() && $('iframe').length && widget.sourceUrl) {
                            window.libTranslateDefer = ZHPromise.defer();
                            var frameUrl = widget.sourceUrl.split('?')[0];
                            $('iframe')[0].contentWindow.postMessage({ type: 'updateTranslation' }, frameUrl);
                            window.libTranslateDefer.then(function() {
                                reloadUpdateTranslation();
                            }).catch(function(error) {
                                ScreenService.hideLoader();
                                LibUtilities.showErrorMessage(error);
                            }); 
                        } else {
                            reloadUpdateTranslation();
                        }
                    }).catch(function(error) {
                        ScreenService.hideLoader();
                        LibUtilities.showErrorMessage(error);
                    });
                },
                fail: function () {}
            }
        );
    };

    var reloadUpdateTranslation = function() {
        MessageService.showMessageByCode('lib#SUC_UPDATE_TRANSLATION');
        setTimeout(function() {
            window.location.reload(true);
        }, 7000);
    };

    this.checkContainerIsZhNext = function() {
        if(templateManager.container.widgets && templateManager.container.widgets.length) {
            return templateManager.container.widgets[0].template == 'widget/iframe.html';
        }
        return false;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});

// FILE: js/Controller/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
			var favoritesText = ScreenService.i18n('Favoritos');
			var favoritesAddText = ScreenService.i18n('Adicionar aos Favoritos');
			var favoritesRemoveText = ScreenService.i18n('Remover dos Favoritos');
			var favoritesUndoText = ScreenService.i18n('Desfazer');
			    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
			    
		    function createVisualElements() {
				var headerContainer = $(element).children()[0];
				headerContainer.style.width = 'calc(100% - 50px)';
				
				favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
				
				var favoritesButtonElement = $compile(
					'<div class="zh-favorites-button" ng-click="toggleList()">'+
					'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
					'    '+startHollowCode+
					'  </div>'+
					'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
					'    '+startFilledCode+
					'  </div>'+
					'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
					'</div>'
				)($scope);
				var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
	    			
				var favoritesListElement = $compile(
					'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
					'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
					'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
					'  <div class="zh-favorites-list-undo" ng-show="false" ng-click="undo()">'+favoritesUndoText+'</div>'+
					'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
					'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
					'      {{ item.label }}'+
					'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
					'    </div>'+
					'  </div>'+
					'</div>'
				)($scope);
				var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
			    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex(function(item) { return item.id === containerId }) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
			    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: ScreenService.i18n(templateManager.container.label),
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'add' }});
		    }
			    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex(function(item) { return item.id === templateManager.container.id });
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'delete' }});
		    	}
		    }
			    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    }
			    
		    var undoTimeout;
			    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = { item: item, index: index };
	
		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(function() {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);
	
			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item: item, status: 'delete' }});
	
		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}
	
		    	event.stopImmediatePropagation();
		    }
			    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    }
	
		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    }
			    
		    document.addEventListener('mousedown', function (evt) {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
			    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, function (args) {
		    	checkIsFavorited(args.container.id);
		    });
			    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', function (favorites) {
		    	favorites = JSON.parse(favorites);
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });
	
		    createVisualElements();
		}
	}
});

// FILE: js/Listener/ZdNext.js
function LibZdNextListener(ScreenService, LibLoginService, LibUtilities, templateManager) {
    window.addEventListener('message', handleMessageEvent);

    function handleMessageEvent(event) {
        if(event.data.type === 'logoutByZdNext') {
            logoutByZdNext()
        } else if (event.data.type === 'versionInfo') {
            var versionInfo = event.data.data.versionInfo;
            if(versionInfo && templateManager.versionData && templateManager.versionData.dependencies) {
                templateManager.versionData.name = versionInfo.name;
                templateManager.versionData.version = versionInfo.version;
                templateManager.versionData.dependencies.backend = versionInfo.backend;
                templateManager.versionData.dependencies.frontend = versionInfo.frontend;
                templateManager.versionData.dependencies.modules = versionInfo.modules;
                templateManager.versionData.dependencies.changeCurrentVersion('frontend');
                templateManager.updateTemplate();
              }
        } else if (event.data.type === 'updateTranslationNext') {
            if(event.data.data.isError) {
                window.libTranslateDefer.reject(event.data.data.message);
            } else {
                window.libTranslateDefer.resolve();
            }
        }
    }

    var logoutByZdNext = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibZdNextListener', LibZdNextListener);
});
